
# 11 "parsing/g_prim.mlg"
 

open Names
open Libnames

open Pcoq.Prim

let prim_kw = ["{"; "}"; "["; "]"; "("; ")"; "'"; "%"; "|"]
let _ = List.iter CLexer.add_keyword prim_kw


let local_make_qualid loc l id = make_qualid ~loc (DirPath.make l) id

let check_int loc = function
  | { NumTok.int = i; frac = ""; exp = "" } -> i
  | _ -> CErrors.user_err ~loc (Pp.str "This number is not an integer.")

let my_int_of_string loc s =
  try
    int_of_string s
  with Failure _ ->
    CErrors.user_err ~loc (Pp.str "This number is too large.")

let rec contiguous tok n m =
    n == m
  ||
    let (_, ep) = Loc.unloc (tok n) in
    let (bp, _) = Loc.unloc (tok (n + 1)) in
    Int.equal ep bp && contiguous tok (succ n) m

let rec lookahead_kwds strm n = function
  | [] -> ()
  | x :: xs ->
      let toks = Stream.npeek (n+1) strm in
      match List.nth toks n with
      | Tok.KEYWORD y ->
          if String.equal x y then lookahead_kwds strm (succ n) xs
          else raise Stream.Failure
      | _ -> raise Stream.Failure
      | exception (Failure _) -> raise Stream.Failure

(* [test_nospace m] fails if the next m tokens are not contiguous keywords *)
let test_nospace m = assert(m <> []); Pcoq.Entry.of_parser "test_nospace"
   (fun tok strm ->
      let n = Stream.count strm in
      lookahead_kwds strm 0 m;
      if contiguous tok n (n + List.length m - 1) then ()
      else raise Stream.Failure)

let test_nospace_pipe_closedcurly =
  test_nospace ["|"; "}"]




let _ = let field = Pcoq.Entry.create "field"
        and fields = Pcoq.Entry.create "fields"
        and basequalid = Pcoq.Entry.create "basequalid"
        in
        let () =
        Pcoq.grammar_extend preident None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PIDENT (None)))),
                 (fun s loc -> 
# 73 "parsing/g_prim.mlg"
                       s 
                               ))])])
        in let () =
        Pcoq.grammar_extend ident None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PIDENT (None)))),
                 (fun s loc -> 
# 76 "parsing/g_prim.mlg"
                       Id.of_string s 
                               ))])])
        in let () =
        Pcoq.grammar_extend pattern_ident None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PLEFTQMARK))),
                              (Extend.Aentry ident)),
                 (fun id _ loc -> 
# 79 "parsing/g_prim.mlg"
                                   id 
                                  ))])])
        in let () =
        Pcoq.grammar_extend pattern_identref None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry pattern_ident)),
                 (fun id loc -> 
# 82 "parsing/g_prim.mlg"
                                CAst.make ~loc id 
                                ))])])
        in let () =
        Pcoq.grammar_extend var None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ident)),
                 (fun id loc -> 
# 85 "parsing/g_prim.mlg"
                        CAst.make ~loc id 
                                ))])])
        in let () =
        Pcoq.grammar_extend identref None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ident)),
                 (fun id loc -> 
# 88 "parsing/g_prim.mlg"
                        CAst.make ~loc id 
                                ))])])
        in let () =
        Pcoq.grammar_extend field None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PFIELD (None)))),
                 (fun s loc -> 
# 91 "parsing/g_prim.mlg"
                       Id.of_string s 
                               ))])])
        in let () =
        Pcoq.grammar_extend fields None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry field)),
                 (fun id loc -> 
# 95 "parsing/g_prim.mlg"
                        ([],id) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry field)),
                             (Extend.Aentry fields)),
                (fun f id loc -> 
# 94 "parsing/g_prim.mlg"
                                    let (l,id') = f in (l@[id],id') 
                                 ))])])
        in let () =
        Pcoq.grammar_extend fullyqualid None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ident)),
                 (fun id loc -> 
# 100 "parsing/g_prim.mlg"
                        CAst.make ~loc [id] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry ident)),
                             (Extend.Aentry fields)),
                (fun f id loc -> 
# 99 "parsing/g_prim.mlg"
                                  let (l,id') = f in CAst.make ~loc @@ id::List.rev (id'::l) 
                                 ))])])
        in let () =
        Pcoq.grammar_extend basequalid None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ident)),
                 (fun id loc -> 
# 105 "parsing/g_prim.mlg"
                        qualid_of_ident ~loc id 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry ident)),
                             (Extend.Aentry fields)),
                (fun f id loc -> 
# 104 "parsing/g_prim.mlg"
                                  let (l,id') = f in  local_make_qualid loc (l@[id]) id' 
                                 ))])])
        in let () =
        Pcoq.grammar_extend name None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ident)),
                 (fun id loc -> 
# 110 "parsing/g_prim.mlg"
                        CAst.make ~loc @@ Name id 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("_"))))),
                (fun _ loc -> 
# 109 "parsing/g_prim.mlg"
                        CAst.make ~loc Anonymous 
                              ))])])
        in let () =
        Pcoq.grammar_extend reference None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ident)),
                 (fun id loc -> 
# 116 "parsing/g_prim.mlg"
                        local_make_qualid loc [] id 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry ident)),
                             (Extend.Aentry fields)),
                (fun f id loc -> 
# 113 "parsing/g_prim.mlg"
                                   
        let (l,id') = f in
        local_make_qualid loc (l@[id]) id' 
                                 ))])])
        in let () =
        Pcoq.grammar_extend by_notation None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry ne_string)),
                              (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("%")))),
                                                          (Extend.Atoken (Tok.PIDENT (None)))),
                                                          (fun key _ loc ->
                                                          
# 120 "parsing/g_prim.mlg"
                                                       key 
                                                          ))]))),
                 (fun sc s loc -> 
# 120 "parsing/g_prim.mlg"
                                                                    (s, sc) 
                                  ))])])
        in let () =
        Pcoq.grammar_extend smart_global None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry by_notation)),
                 (fun ntn loc -> 
# 124 "parsing/g_prim.mlg"
                               CAst.make ~loc @@ Constrexpr.ByNotation ntn 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry reference)),
                (fun c loc -> 
# 123 "parsing/g_prim.mlg"
                           CAst.make ~loc @@ Constrexpr.AN c 
                              ))])])
        in let () =
        Pcoq.grammar_extend qualid None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry basequalid)),
                 (fun qid loc -> 
# 127 "parsing/g_prim.mlg"
                              qid 
                                 ))])])
        in let () =
        Pcoq.grammar_extend ne_string None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PSTRING (None)))),
                 (fun s loc -> 
# 131 "parsing/g_prim.mlg"
          if s="" then CErrors.user_err ~loc (Pp.str"Empty string."); s 
                               ))])])
        in let () =
        Pcoq.grammar_extend ne_lstring None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ne_string)),
                 (fun s loc -> 
# 135 "parsing/g_prim.mlg"
                           CAst.make ~loc s 
                               ))])])
        in let () =
        Pcoq.grammar_extend dirpath None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry ident)),
                              (Extend.Alist0 (Extend.Aentry field))),
                 (fun l id loc -> 
# 139 "parsing/g_prim.mlg"
          DirPath.make (List.rev (id::l)) 
                                  ))])])
        in let () =
        Pcoq.grammar_extend string None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PSTRING (None)))),
                 (fun s loc -> 
# 142 "parsing/g_prim.mlg"
                        s 
                               ))])])
        in let () =
        Pcoq.grammar_extend lstring None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry string)),
                 (fun s loc -> 
# 145 "parsing/g_prim.mlg"
                        CAst.make ~loc s 
                               ))])])
        in let () =
        Pcoq.grammar_extend integer None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("-")))),
                              (Extend.Atoken (Tok.PNUMERAL None))),
                 (fun i _ loc -> 
# 149 "parsing/g_prim.mlg"
                              - my_int_of_string loc (check_int loc i) 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PNUMERAL None))),
                (fun i loc -> 
# 148 "parsing/g_prim.mlg"
                              my_int_of_string loc (check_int loc i) 
                              ))])])
        in let () =
        Pcoq.grammar_extend natural None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PNUMERAL None))),
                 (fun i loc -> 
# 152 "parsing/g_prim.mlg"
                         my_int_of_string loc (check_int loc i) 
                               ))])])
        in let () =
        Pcoq.grammar_extend bigint None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PNUMERAL None))),
                 (fun i loc -> 
# 155 "parsing/g_prim.mlg"
                         check_int loc i 
                               ))])])
        in let () =
        Pcoq.grammar_extend bar_cbrace None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aentry test_nospace_pipe_closedcurly)),
                                           (Extend.Atoken (Tok.PKEYWORD ("|")))),
                              (Extend.Atoken (Tok.PKEYWORD ("}")))),
                 (fun _ _ _ loc -> 
# 158 "parsing/g_prim.mlg"
                                                     () 
                                   ))])])
        in ()

