
# 11 "vernac/g_vernac.mlg"
 

open Pp
open CErrors
open Util
open Names
open Glob_term
open Vernacexpr
open Impargs
open Constrexpr
open Constrexpr_ops
open Extend
open Decls
open Declaremods
open Declarations
open Namegen
open Tok (* necessary for camlp5 *)

open Pcoq
open Pcoq.Prim
open Pcoq.Constr
open Pcoq.Module
open Pvernac.Vernac_
open Attributes

let vernac_kw = [ ";"; ","; ">->"; ":<"; "<:"; "where"; "at" ]
let _ = List.iter CLexer.add_keyword vernac_kw

(* Rem: do not join the different GEXTEND into one, it breaks native *)
(* compilation on PowerPC and Sun architectures *)

let query_command = Entry.create "vernac:query_command"

let subprf = Entry.create "vernac:subprf"

let quoted_attributes = Entry.create "vernac:quoted_attributes"
let class_rawexpr = Entry.create "vernac:class_rawexpr"
let thm_token = Entry.create "vernac:thm_token"
let def_body = Entry.create "vernac:def_body"
let decl_notation = Entry.create "vernac:decl_notation"
let record_field = Entry.create "vernac:record_field"
let of_type_with_opt_coercion = Entry.create "vernac:of_type_with_opt_coercion"
let section_subset_expr = Entry.create "vernac:section_subset_expr"

let make_bullet s =
  let open Proof_bullet in
  let n = String.length s in
  match s.[0] with
  | '-' -> Dash n
  | '+' -> Plus n
  | '*' -> Star n
  | _ -> assert false

let parse_compat_version = let open Flags in function
  | "8.11" -> Current
  | "8.10" -> V8_10
  | "8.9" -> V8_9
  | ("8.8" | "8.7" | "8.6" | "8.5" | "8.4" | "8.3" | "8.2" | "8.1" | "8.0") as s ->
    CErrors.user_err ~hdr:"get_compat_version"
      Pp.(str "Compatibility with version " ++ str s ++ str " not supported.")
  | s ->
    CErrors.user_err ~hdr:"get_compat_version"
      Pp.(str "Unknown compatibility version \"" ++ str s ++ str "\".")

(* For now we just keep the top-level location of the whole
   vernacular, that is to say, including attributes and control flags;
   this is not very convenient for advanced clients tho, so in the
   future it'd be cool to actually locate the attributes and control
   flags individually too. *)
let add_control_flag ~loc ~flag { CAst.v = cmd } =
  CAst.make ~loc { cmd with control = flag :: cmd.control }



let _ = let decorated_vernac = Pcoq.Entry.create "decorated_vernac"
        and attribute_list = Pcoq.Entry.create "attribute_list"
        and attribute = Pcoq.Entry.create "attribute"
        and attribute_value = Pcoq.Entry.create "attribute_value"
        and vernac = Pcoq.Entry.create "vernac"
        and vernac_poly = Pcoq.Entry.create "vernac_poly"
        and vernac_aux = Pcoq.Entry.create "vernac_aux"
        in
        let () =
        Pcoq.grammar_extend vernac_control None
        (Some
        (Gramlib.Gramext.First), [(None, None,
                                  [Extend.Rule
                                   (Extend.Next (Extend.Stop,
                                                (Extend.Aentry decorated_vernac)),
                                   (fun v loc -> 
# 97 "vernac/g_vernac.mlg"
          let (attrs, expr) = v in CAst.make ~loc { control = []; attrs; expr = expr } 
                                                 ));
                                  Extend.Rule
                                  (Extend.Next (Extend.Next (Extend.Stop,
                                                            (Extend.Atoken (Tok.PIDENT (Some
                                                            ("Fail"))))),
                                               (Extend.Aentry vernac_control)),
                                  (fun c _ loc -> 
# 95 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:ControlFail c 
                                                  ));
                                  Extend.Rule
                                  (Extend.Next (Extend.Next (Extend.Next 
                                                            (Extend.Stop,
                                                            (Extend.Atoken (Tok.PIDENT (Some
                                                            ("Timeout"))))),
                                                            (Extend.Aentry natural)),
                                               (Extend.Aentry vernac_control)),
                                  (fun c n _ loc -> 
# 93 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:(ControlTimeout n) c 
                                                    ));
                                  Extend.Rule
                                  (Extend.Next (Extend.Next (Extend.Next 
                                                            (Extend.Stop,
                                                            (Extend.Atoken (Tok.PIDENT (Some
                                                            ("Redirect"))))),
                                                            (Extend.Aentry ne_string)),
                                               (Extend.Aentry vernac_control)),
                                  (fun c s _ loc -> 
# 91 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:(ControlRedirect s) c 
                                                    ));
                                  Extend.Rule
                                  (Extend.Next (Extend.Next (Extend.Stop,
                                                            (Extend.Atoken (Tok.PIDENT (Some
                                                            ("Time"))))),
                                               (Extend.Aentry vernac_control)),
                                  (fun c _ loc -> 
# 89 "vernac/g_vernac.mlg"
          add_control_flag ~loc ~flag:(ControlTime false) c 
                                                  ))])])
        in let () =
        Pcoq.grammar_extend decorated_vernac None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Alist0 (Extend.Aentry quoted_attributes))),
                              (Extend.Aentry vernac)),
                 (fun fv a loc -> 
# 102 "vernac/g_vernac.mlg"
          let (f, v) = fv in (List.append (List.flatten a) f, v) 
                                  ))])])
        in let () =
        Pcoq.grammar_extend quoted_attributes None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("#[")))),
                                           (Extend.Aentry attribute_list)),
                              (Extend.Atoken (Tok.PKEYWORD ("]")))),
                 (fun _ a _ loc -> 
# 105 "vernac/g_vernac.mlg"
                                             a 
                                   ))])])
        in let () =
        Pcoq.grammar_extend attribute_list None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Alist0sep ((Extend.Aentry attribute), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                 (fun a loc -> 
# 109 "vernac/g_vernac.mlg"
                                         a 
                               ))])])
        in let () =
        Pcoq.grammar_extend attribute None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry ident)),
                              (Extend.Aentry attribute_value)),
                 (fun v k loc -> 
# 113 "vernac/g_vernac.mlg"
                                             Names.Id.to_string k, v 
                                 ))])])
        in let () =
        Pcoq.grammar_extend attribute_value None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 119 "vernac/g_vernac.mlg"
             VernacFlagEmpty 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                          (Extend.Aentry attribute_list)),
                             (Extend.Atoken (Tok.PKEYWORD (")")))),
                (fun _ a _ loc -> 
# 118 "vernac/g_vernac.mlg"
                                            VernacFlagList a 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("=")))),
                             (Extend.Aentry string)),
                (fun v _ loc -> 
# 117 "vernac/g_vernac.mlg"
                              VernacFlagLeaf v 
                                ))])])
        in let () =
        Pcoq.grammar_extend vernac None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry vernac_poly)),
                 (fun v loc -> 
# 126 "vernac/g_vernac.mlg"
                             v 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Global"))))),
                             (Extend.Aentry vernac_poly)),
                (fun v _ loc -> 
# 124 "vernac/g_vernac.mlg"
                                             let (f, v) = v in (("global", VernacFlagEmpty) :: f, v) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Local"))))),
                             (Extend.Aentry vernac_poly)),
                (fun v _ loc -> 
# 123 "vernac/g_vernac.mlg"
                                            let (f, v) = v in (("local", VernacFlagEmpty) :: f, v) 
                                ))])])
        in let () =
        Pcoq.grammar_extend vernac_poly None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry vernac_aux)),
                 (fun v loc -> 
# 134 "vernac/g_vernac.mlg"
                            v 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Monomorphic"))))),
                             (Extend.Aentry vernac_aux)),
                (fun v _ loc -> 
# 133 "vernac/g_vernac.mlg"
          let (f, v) = v in (Attributes.vernac_monomorphic_flag :: f, v) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Polymorphic"))))),
                             (Extend.Aentry vernac_aux)),
                (fun v _ loc -> 
# 131 "vernac/g_vernac.mlg"
          let (f, v) = v in (Attributes.vernac_polymorphic_flag :: f, v) 
                                ))])])
        in let () =
        Pcoq.grammar_extend vernac_aux None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry subprf)),
                 (fun c loc -> 
# 146 "vernac/g_vernac.mlg"
                        ([], c) 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry syntax)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ c loc -> 
# 145 "vernac/g_vernac.mlg"
                             ([], c) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry command)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ c loc -> 
# 144 "vernac/g_vernac.mlg"
                              ([], c) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry gallina_ext)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ g loc -> 
# 143 "vernac/g_vernac.mlg"
                                  ([], g) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry gallina)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ g loc -> 
# 142 "vernac/g_vernac.mlg"
                              ([], g) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Program"))))),
                                          (Extend.Aentry gallina_ext)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ g _ loc -> 
# 141 "vernac/g_vernac.mlg"
                                                   (["program", VernacFlagEmpty], g) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Program"))))),
                                          (Extend.Aentry gallina)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ g _ loc -> 
# 140 "vernac/g_vernac.mlg"
                                               (["program", VernacFlagEmpty], g) 
                                  ))])])
        in let () =
        Pcoq.grammar_extend vernac_aux None
        (Some
        (Gramlib.Gramext.Last), [(None, None,
                                 [Extend.Rule
                                  (Extend.Next (Extend.Stop,
                                               (Extend.Aentry command_entry)),
                                  (fun prfcom loc -> 
# 150 "vernac/g_vernac.mlg"
                                    ([], prfcom) 
                                                     ))])])
        in let () =
        Pcoq.grammar_extend noedit_mode None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry query_command)),
                 (fun c loc -> 
# 153 "vernac/g_vernac.mlg"
                               c None 
                               ))])])
        in let () =
        Pcoq.grammar_extend subprf None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PKEYWORD ("}")))),
                 (fun _ loc -> 
# 159 "vernac/g_vernac.mlg"
               VernacEndSubproof 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("{")))),
                (fun _ loc -> 
# 158 "vernac/g_vernac.mlg"
               VernacSubproof None 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PBULLET (None)))),
                (fun s loc -> 
# 157 "vernac/g_vernac.mlg"
                      VernacBullet (make_bullet s) 
                              ))])])
        in ()


# 165 "vernac/g_vernac.mlg"
 

let warn_plural_command =
  CWarnings.create ~name:"plural-command" ~category:"pedantic" ~default:CWarnings.Disabled
         (fun kwd -> strbrk (Printf.sprintf "Command \"%s\" expects more than one assumption." kwd))

let test_plural_form loc kwd = function
  | [(_,([_],_))] ->
     warn_plural_command ~loc kwd
  | _ -> ()

let test_plural_form_types loc kwd = function
  | [([_],_)] ->
     warn_plural_command ~loc kwd
  | _ -> ()

let lname_of_lident : lident -> lname =
  CAst.map (fun s -> Name s)

let name_of_ident_decl : ident_decl -> name_decl =
  on_fst lname_of_lident



let _ = let register_token = Pcoq.Entry.create "register_token"
        and register_type_token = Pcoq.Entry.create "register_type_token"
        and register_prim_token = Pcoq.Entry.create "register_prim_token"
        and def_token = Pcoq.Entry.create "def_token"
        and assumption_token = Pcoq.Entry.create "assumption_token"
        and assumptions_token = Pcoq.Entry.create "assumptions_token"
        and inline = Pcoq.Entry.create "inline"
        and univ_constraint = Pcoq.Entry.create "univ_constraint"
        and finite_token = Pcoq.Entry.create "finite_token"
        and cumulativity_token = Pcoq.Entry.create "cumulativity_token"
        and private_token = Pcoq.Entry.create "private_token"
        and reduce = Pcoq.Entry.create "reduce"
        and one_decl_notation = Pcoq.Entry.create "one_decl_notation"
        and decl_sep = Pcoq.Entry.create "decl_sep"
        and opt_constructors_or_fields =
          Pcoq.Entry.create "opt_constructors_or_fields"
        and inductive_definition = Pcoq.Entry.create "inductive_definition"
        and constructor_list_or_record_decl =
          Pcoq.Entry.create "constructor_list_or_record_decl"
        and opt_coercion = Pcoq.Entry.create "opt_coercion"
        and corec_definition = Pcoq.Entry.create "corec_definition"
        and rec_type_cstr = Pcoq.Entry.create "rec_type_cstr"
        and scheme = Pcoq.Entry.create "scheme"
        and scheme_kind = Pcoq.Entry.create "scheme_kind"
        and record_fields = Pcoq.Entry.create "record_fields"
        and record_binder_body = Pcoq.Entry.create "record_binder_body"
        and record_binder = Pcoq.Entry.create "record_binder"
        and assum_list = Pcoq.Entry.create "assum_list"
        and assum_coe = Pcoq.Entry.create "assum_coe"
        and simple_assum_coe = Pcoq.Entry.create "simple_assum_coe"
        and constructor_type = Pcoq.Entry.create "constructor_type"
        and constructor = Pcoq.Entry.create "constructor"
        in
        let () =
        Pcoq.grammar_extend gallina None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("Constraint"))))),
                              (Extend.Alist1sep ((Extend.Aentry univ_constraint), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                 (fun l _ loc -> 
# 236 "vernac/g_vernac.mlg"
                                                                   VernacConstraint l 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Universes"))))),
                             (Extend.Alist1 (Extend.Aentry identref))),
                (fun l _ loc -> 
# 235 "vernac/g_vernac.mlg"
                                                   VernacUniverse l 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Universe"))))),
                             (Extend.Alist1 (Extend.Aentry identref))),
                (fun l _ loc -> 
# 234 "vernac/g_vernac.mlg"
                                                  VernacUniverse l 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Primitive"))))),
                                                                    (Extend.Aentry identref)),
                                                       (Extend.Aopt (Extend.Arules 
                                                       [Extend.Rules 
                                                       (Extend.NextNoRec 
                                                       (Extend.NextNoRec 
                                                       (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                       (Extend.Aentry lconstr)),
                                                       (fun typ _ loc -> 
                                                       
# 232 "vernac/g_vernac.mlg"
                                                                                 typ 
                                                       ))]))),
                                          (Extend.Atoken (Tok.PKEYWORD (":=")))),
                             (Extend.Aentry register_token)),
                (fun r _ typopt id _ loc -> 
# 233 "vernac/g_vernac.mlg"
            VernacPrimitive(id, r, typopt) 
                                            ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Register"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Inline"))))),
                             (Extend.Aentry global)),
                (fun g _ _ loc -> 
# 231 "vernac/g_vernac.mlg"
            VernacRegister(g, RegisterInline) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Register"))))),
                                                       (Extend.Aentry global)),
                                          (Extend.Atoken (Tok.PKEYWORD ("as")))),
                             (Extend.Aentry qualid)),
                (fun quid _ g _ loc -> 
# 229 "vernac/g_vernac.mlg"
            VernacRegister(g, RegisterCoqlib quid) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Combined"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Scheme"))))),
                                                       (Extend.Aentry identref)),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("from"))))),
                             (Extend.Alist1sep ((Extend.Aentry identref), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                (fun l _ id _ _ loc -> 
# 227 "vernac/g_vernac.mlg"
                                              VernacCombinedScheme (id, l) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Scheme"))))),
                             (Extend.Alist1sep ((Extend.Aentry scheme), (Extend.Atoken (Tok.PKEYWORD ("with")))))),
                (fun l _ loc -> 
# 225 "vernac/g_vernac.mlg"
                                                         VernacScheme l 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Let"))))),
                                          (Extend.Atoken (Tok.PKEYWORD ("CoFixpoint")))),
                             (Extend.Alist1sep ((Extend.Aentry corec_definition), (Extend.Atoken (Tok.PKEYWORD ("with")))))),
                (fun corecs _ _ loc -> 
# 224 "vernac/g_vernac.mlg"
            VernacCoFixpoint (DoDischarge, corecs) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("CoFixpoint")))),
                             (Extend.Alist1sep ((Extend.Aentry corec_definition), (Extend.Atoken (Tok.PKEYWORD ("with")))))),
                (fun corecs _ loc -> 
# 222 "vernac/g_vernac.mlg"
            VernacCoFixpoint (NoDischarge, corecs) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Let"))))),
                                          (Extend.Atoken (Tok.PKEYWORD ("Fixpoint")))),
                             (Extend.Alist1sep ((Extend.Aentry rec_definition), (Extend.Atoken (Tok.PKEYWORD ("with")))))),
                (fun recs _ _ loc -> 
# 220 "vernac/g_vernac.mlg"
            VernacFixpoint (DoDischarge, recs) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("Fixpoint")))),
                             (Extend.Alist1sep ((Extend.Aentry rec_definition), (Extend.Atoken (Tok.PKEYWORD ("with")))))),
                (fun recs _ loc -> 
# 218 "vernac/g_vernac.mlg"
            VernacFixpoint (NoDischarge, recs) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Aopt (Extend.Aentry cumulativity_token))),
                                                       (Extend.Aentry private_token)),
                                          (Extend.Aentry finite_token)),
                             (Extend.Alist1sep ((Extend.Aentry inductive_definition), (Extend.Atoken (Tok.PKEYWORD ("with")))))),
                (fun indl f priv cum loc -> 
# 214 "vernac/g_vernac.mlg"
            let (k,f) = f in
          let indl=List.map (fun ((a,b,c,d),e) -> ((a,b,c,k,d),e)) indl in
          VernacInductive (cum, priv,f,indl) 
                                            ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Let"))))),
                                          (Extend.Aentry identref)),
                             (Extend.Aentry def_body)),
                (fun b id _ loc -> 
# 210 "vernac/g_vernac.mlg"
            VernacDefinition ((DoDischarge, Let), (lname_of_lident id, None), b) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry def_token)),
                                          (Extend.Aentry ident_decl)),
                             (Extend.Aentry def_body)),
                (fun b id d loc -> 
# 208 "vernac/g_vernac.mlg"
            VernacDefinition (d, name_of_ident_decl id, b) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry assumptions_token)),
                                          (Extend.Aentry inline)),
                             (Extend.Aentry assum_list)),
                (fun bl nl tk loc -> 
# 204 "vernac/g_vernac.mlg"
            let (kwd,stre) = tk in
            test_plural_form loc kwd bl;
            VernacAssumption (stre, nl, bl) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry assumption_token)),
                                          (Extend.Aentry inline)),
                             (Extend.Aentry assum_list)),
                (fun bl nl stre loc -> 
# 202 "vernac/g_vernac.mlg"
            VernacAssumption (stre, nl, bl) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Aentry thm_token)),
                                                                    (Extend.Aentry ident_decl)),
                                                                    (Extend.Aentry binders)),
                                                       (Extend.Atoken (Tok.PKEYWORD (":")))),
                                          (Extend.Aentry lconstr)),
                             (Extend.Alist0 (Extend.Arules [Extend.Rules 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Atoken (Tok.PKEYWORD ("with")))),
                                                           (Extend.Aentry ident_decl)),
                                                           (Extend.Aentry binders)),
                                                           (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                           (Extend.Aentry lconstr)),
                                                           (fun c _ bl id _
                                                           loc -> 
# 199 "vernac/g_vernac.mlg"
            (id,(bl,c)) 
                                                                  ))]))),
                (fun l c _ bl id thm loc -> 
# 200 "vernac/g_vernac.mlg"
            VernacStartTheoremProof (thm, (id,(bl,c))::l) 
                                            ))])])
        in let () =
        Pcoq.grammar_extend register_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Aentry register_type_token)),
                 (fun r loc -> 
# 242 "vernac/g_vernac.mlg"
                                      CPrimitives.OT_type r 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Aentry register_prim_token)),
                (fun r loc -> 
# 241 "vernac/g_vernac.mlg"
                                      CPrimitives.OT_op r 
                              ))])])
        in let () =
        Pcoq.grammar_extend register_type_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PKEYWORD ("#float64_type")))),
                 (fun _ loc -> 
# 247 "vernac/g_vernac.mlg"
                              CPrimitives.PT_float64 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_type")))),
                (fun _ loc -> 
# 246 "vernac/g_vernac.mlg"
                            CPrimitives.PT_int63 
                              ))])])
        in let () =
        Pcoq.grammar_extend register_prim_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PKEYWORD ("#float64_next_down")))),
                 (fun _ loc -> 
# 292 "vernac/g_vernac.mlg"
                                   CPrimitives.Float64next_down 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_next_up")))),
                (fun _ loc -> 
# 291 "vernac/g_vernac.mlg"
                                 CPrimitives.Float64next_up 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_ldshiftexp")))),
                (fun _ loc -> 
# 290 "vernac/g_vernac.mlg"
                                    CPrimitives.Float64ldshiftexp 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_frshiftexp")))),
                (fun _ loc -> 
# 289 "vernac/g_vernac.mlg"
                                    CPrimitives.Float64frshiftexp 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_normfr_mantissa")))),
                (fun _ loc -> 
# 288 "vernac/g_vernac.mlg"
                                         CPrimitives.Float64normfr_mantissa 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_of_int63")))),
                (fun _ loc -> 
# 287 "vernac/g_vernac.mlg"
                                  CPrimitives.Float64ofInt63 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_sqrt")))),
                (fun _ loc -> 
# 286 "vernac/g_vernac.mlg"
                              CPrimitives.Float64sqrt 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_div")))),
                (fun _ loc -> 
# 285 "vernac/g_vernac.mlg"
                             CPrimitives.Float64div 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_mul")))),
                (fun _ loc -> 
# 284 "vernac/g_vernac.mlg"
                             CPrimitives.Float64mul 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_sub")))),
                (fun _ loc -> 
# 283 "vernac/g_vernac.mlg"
                             CPrimitives.Float64sub 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_add")))),
                (fun _ loc -> 
# 282 "vernac/g_vernac.mlg"
                             CPrimitives.Float64add 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_classify")))),
                (fun _ loc -> 
# 281 "vernac/g_vernac.mlg"
                                  CPrimitives.Float64classify 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_compare")))),
                (fun _ loc -> 
# 280 "vernac/g_vernac.mlg"
                                 CPrimitives.Float64compare 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_le")))),
                (fun _ loc -> 
# 279 "vernac/g_vernac.mlg"
                            CPrimitives.Float64le 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_lt")))),
                (fun _ loc -> 
# 278 "vernac/g_vernac.mlg"
                            CPrimitives.Float64lt 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_eq")))),
                (fun _ loc -> 
# 277 "vernac/g_vernac.mlg"
                            CPrimitives.Float64eq 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_abs")))),
                (fun _ loc -> 
# 276 "vernac/g_vernac.mlg"
                             CPrimitives.Float64abs 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#float64_opp")))),
                (fun _ loc -> 
# 275 "vernac/g_vernac.mlg"
                             CPrimitives.Float64opp 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_compare")))),
                (fun _ loc -> 
# 274 "vernac/g_vernac.mlg"
                               CPrimitives.Int63compare 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_le")))),
                (fun _ loc -> 
# 273 "vernac/g_vernac.mlg"
                          CPrimitives.Int63le 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_lt")))),
                (fun _ loc -> 
# 272 "vernac/g_vernac.mlg"
                          CPrimitives.Int63lt 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_eq")))),
                (fun _ loc -> 
# 271 "vernac/g_vernac.mlg"
                          CPrimitives.Int63eq 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_addmuldiv")))),
                (fun _ loc -> 
# 270 "vernac/g_vernac.mlg"
                                 CPrimitives.Int63addMulDiv 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_div21")))),
                (fun _ loc -> 
# 269 "vernac/g_vernac.mlg"
                             CPrimitives.Int63div21 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_diveucl")))),
                (fun _ loc -> 
# 268 "vernac/g_vernac.mlg"
                               CPrimitives.Int63diveucl 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_mulc")))),
                (fun _ loc -> 
# 267 "vernac/g_vernac.mlg"
                            CPrimitives.Int63mulc 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_subcarryc")))),
                (fun _ loc -> 
# 266 "vernac/g_vernac.mlg"
                                 CPrimitives.Int63subCarryC 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_addcarryc")))),
                (fun _ loc -> 
# 265 "vernac/g_vernac.mlg"
                                 CPrimitives.Int63addCarryC 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_subc")))),
                (fun _ loc -> 
# 264 "vernac/g_vernac.mlg"
                            CPrimitives.Int63subc 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_addc")))),
                (fun _ loc -> 
# 263 "vernac/g_vernac.mlg"
                            CPrimitives.Int63addc 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_lxor")))),
                (fun _ loc -> 
# 262 "vernac/g_vernac.mlg"
                            CPrimitives.Int63lxor 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_lor")))),
                (fun _ loc -> 
# 261 "vernac/g_vernac.mlg"
                           CPrimitives.Int63lor 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_land")))),
                (fun _ loc -> 
# 260 "vernac/g_vernac.mlg"
                            CPrimitives.Int63land 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_lsl")))),
                (fun _ loc -> 
# 259 "vernac/g_vernac.mlg"
                           CPrimitives.Int63lsl 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_lsr")))),
                (fun _ loc -> 
# 258 "vernac/g_vernac.mlg"
                           CPrimitives.Int63lsr 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_mod")))),
                (fun _ loc -> 
# 257 "vernac/g_vernac.mlg"
                           CPrimitives.Int63mod 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_div")))),
                (fun _ loc -> 
# 256 "vernac/g_vernac.mlg"
                           CPrimitives.Int63div 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_mul")))),
                (fun _ loc -> 
# 255 "vernac/g_vernac.mlg"
                           CPrimitives.Int63mul 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_sub")))),
                (fun _ loc -> 
# 254 "vernac/g_vernac.mlg"
                           CPrimitives.Int63sub 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_add")))),
                (fun _ loc -> 
# 253 "vernac/g_vernac.mlg"
                           CPrimitives.Int63add 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_tail0")))),
                (fun _ loc -> 
# 252 "vernac/g_vernac.mlg"
                             CPrimitives.Int63tail0 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("#int63_head0")))),
                (fun _ loc -> 
# 251 "vernac/g_vernac.mlg"
                             CPrimitives.Int63head0 
                              ))])])
        in let () =
        Pcoq.grammar_extend thm_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("Property"))))),
                 (fun _ loc -> 
# 303 "vernac/g_vernac.mlg"
                              Property 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Proposition"))))),
                (fun _ loc -> 
# 302 "vernac/g_vernac.mlg"
                                 Proposition 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Corollary"))))),
                (fun _ loc -> 
# 301 "vernac/g_vernac.mlg"
                               Corollary 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Remark"))))),
                (fun _ loc -> 
# 300 "vernac/g_vernac.mlg"
                            Remark 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Fact"))))),
                (fun _ loc -> 
# 299 "vernac/g_vernac.mlg"
                          Fact 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Lemma"))))),
                (fun _ loc -> 
# 298 "vernac/g_vernac.mlg"
                           Lemma 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Theorem")))),
                (fun _ loc -> 
# 297 "vernac/g_vernac.mlg"
                       Theorem 
                              ))])])
        in let () =
        Pcoq.grammar_extend def_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("SubClass"))))),
                 (fun _ loc -> 
# 308 "vernac/g_vernac.mlg"
                              (NoDischarge,SubClass) 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Example"))))),
                (fun _ loc -> 
# 307 "vernac/g_vernac.mlg"
                             (NoDischarge,Example) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Definition")))),
                (fun _ loc -> 
# 306 "vernac/g_vernac.mlg"
                          (NoDischarge,Definition) 
                              ))])])
        in let () =
        Pcoq.grammar_extend assumption_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("Conjecture"))))),
                 (fun _ loc -> 
# 315 "vernac/g_vernac.mlg"
                                (NoDischarge, Conjectural) 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Parameter")))),
                (fun _ loc -> 
# 314 "vernac/g_vernac.mlg"
                         (NoDischarge, Definitional) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Axiom")))),
                (fun _ loc -> 
# 313 "vernac/g_vernac.mlg"
                     (NoDischarge, Logical) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Variable")))),
                (fun _ loc -> 
# 312 "vernac/g_vernac.mlg"
                        (DoDischarge, Definitional) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Hypothesis")))),
                (fun _ loc -> 
# 311 "vernac/g_vernac.mlg"
                          (DoDischarge, Logical) 
                              ))])])
        in let () =
        Pcoq.grammar_extend assumptions_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("Conjectures"))))),
                 (fun _ loc -> 
# 322 "vernac/g_vernac.mlg"
                                 ("Conjectures", (NoDischarge, Conjectural)) 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Parameters"))))),
                (fun _ loc -> 
# 321 "vernac/g_vernac.mlg"
                                ("Parameters", (NoDischarge, Definitional)) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Axioms"))))),
                (fun _ loc -> 
# 320 "vernac/g_vernac.mlg"
                            ("Axioms", (NoDischarge, Logical)) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Variables"))))),
                (fun _ loc -> 
# 319 "vernac/g_vernac.mlg"
                               ("Variables", (DoDischarge, Definitional)) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Hypotheses"))))),
                (fun _ loc -> 
# 318 "vernac/g_vernac.mlg"
                                ("Hypotheses", (DoDischarge, Logical)) 
                              ))])])
        in let () =
        Pcoq.grammar_extend inline None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 327 "vernac/g_vernac.mlg"
             NoInline 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Inline"))))),
                (fun _ loc -> 
# 326 "vernac/g_vernac.mlg"
                            DefaultInline 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Inline"))))),
                                                       (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                          (Extend.Aentry natural)),
                             (Extend.Atoken (Tok.PKEYWORD (")")))),
                (fun _ i _ _ loc -> 
# 325 "vernac/g_vernac.mlg"
                                                   InlineAt i 
                                    ))])])
        in let () =
        Pcoq.grammar_extend univ_constraint None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aentry universe_name)),
                                           (Extend.Arules [Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("<=")))),
                                                          (fun _ loc -> 
                                                          
# 330 "vernac/g_vernac.mlg"
                                                                                       Univ.Le 
                                                          ));
                                                          Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("=")))),
                                                          (fun _ loc -> 
                                                          
# 330 "vernac/g_vernac.mlg"
                                                                 Univ.Eq 
                                                          ));
                                                          Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("<")))),
                                                          (fun _ loc -> 
                                                          
# 330 "vernac/g_vernac.mlg"
                                            Univ.Lt 
                                                          ))])),
                              (Extend.Aentry universe_name)),
                 (fun r ord l loc -> 
# 331 "vernac/g_vernac.mlg"
                               (l, ord, r) 
                                     ))])])
        in let () =
        Pcoq.grammar_extend univ_decl None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("@{")))),
                                                        (Extend.Alist0 (Extend.Aentry identref))),
                                           (Extend.Arules [Extend.Rules 
                                                          (Extend.Stop, (fun
                                                          loc -> 
# 334 "vernac/g_vernac.mlg"
                                                                   false 
                                                                 ));
                                                          Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("+")))),
                                                          (fun _ loc -> 
                                                          
# 334 "vernac/g_vernac.mlg"
                                                     true 
                                                          ))])),
                              (Extend.Arules [Extend.Rules (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Arules 
                                                           [Extend.Rules 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Aentry bar_cbrace)),
                                                           (fun _ loc -> 
                                                           
# 337 "vernac/g_vernac.mlg"
                                                          false 
                                                           ));
                                                           Extend.Rules 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Atoken (Tok.PKEYWORD ("}")))),
                                                           (fun _ loc -> 
                                                           
# 337 "vernac/g_vernac.mlg"
                                 true 
                                                           ))])), (fun ext
                                                           loc -> 
# 337 "vernac/g_vernac.mlg"
                                                                         ([], ext) 
                                                                  ));
                                             Extend.Rules (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("|")))),
                                                          (Extend.Alist0sep ((Extend.Aentry univ_constraint), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                          (Extend.Arules 
                                                          [Extend.Rules 
                                                          (Extend.Stop, (fun
                                                          loc -> 
# 336 "vernac/g_vernac.mlg"
                                               false 
                                                                 ));
                                                          Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("+")))),
                                                          (fun _ loc -> 
                                                          
# 336 "vernac/g_vernac.mlg"
                                 true 
                                                          ))])),
                                                          (Extend.Atoken (Tok.PKEYWORD ("}")))),
                                                          (fun _ ext l' _
                                                          loc -> 
# 336 "vernac/g_vernac.mlg"
                                                                   (l',ext) 
                                                                 ))])),
                 (fun cs ext l _ loc -> 
# 339 "vernac/g_vernac.mlg"
           let open UState in
         { univdecl_instance = l;
           univdecl_extensible_instance = ext;
           univdecl_constraints = fst cs;
           univdecl_extensible_constraints = snd cs } 
                                        ))])])
        in let () =
        Pcoq.grammar_extend ident_decl None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry identref)),
                              (Extend.Aopt (Extend.Aentry univ_decl))),
                 (fun l i loc -> 
# 347 "vernac/g_vernac.mlg"
                                             (i, l) 
                                 ))])])
        in let () =
        Pcoq.grammar_extend finite_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("Class"))))),
                 (fun _ loc -> 
# 356 "vernac/g_vernac.mlg"
                           (Class true,BiFinite) 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Structure"))))),
                (fun _ loc -> 
# 355 "vernac/g_vernac.mlg"
                               (Structure,BiFinite) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Record"))))),
                (fun _ loc -> 
# 354 "vernac/g_vernac.mlg"
                            (Record,BiFinite) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Variant"))))),
                (fun _ loc -> 
# 353 "vernac/g_vernac.mlg"
                             (Variant,BiFinite) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("CoInductive"))))),
                (fun _ loc -> 
# 352 "vernac/g_vernac.mlg"
                                 (CoInductive,CoFinite) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Inductive"))))),
                (fun _ loc -> 
# 351 "vernac/g_vernac.mlg"
                               (Inductive_kw,Finite) 
                              ))])])
        in let () =
        Pcoq.grammar_extend cumulativity_token None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("NonCumulative"))))),
                 (fun _ loc -> 
# 360 "vernac/g_vernac.mlg"
                                   VernacNonCumulative 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Cumulative"))))),
                (fun _ loc -> 
# 359 "vernac/g_vernac.mlg"
                                VernacCumulative 
                              ))])])
        in let () =
        Pcoq.grammar_extend private_token None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 363 "vernac/g_vernac.mlg"
                                           false 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Private"))))),
                (fun _ loc -> 
# 363 "vernac/g_vernac.mlg"
                             true 
                              ))])])
        in let () =
        Pcoq.grammar_extend def_body None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aentry binders)),
                                           (Extend.Atoken (Tok.PKEYWORD (":")))),
                              (Extend.Aentry lconstr)),
                 (fun t _ bl loc -> 
# 388 "vernac/g_vernac.mlg"
          ProveBody (bl, t) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Aentry binders)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                                    (Extend.Aentry lconstr)),
                                                       (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                          (Extend.Aentry reduce)),
                             (Extend.Aentry lconstr)),
                (fun c red _ t _ bl loc -> 
# 378 "vernac/g_vernac.mlg"
          let ((bl, c), tyo) =
          if List.exists (function CLocalPattern _ -> true | _ -> false) bl
          then
            (* FIXME: "red" will be applied to types in bl and Cast with remain *)
            let c = CAst.make ~loc @@ CCast (c, CastConv t) in
            (([],mkLambdaCN ~loc bl c), None)
          else ((bl, c), Some t)
        in
        DefineBody (bl, red, c, tyo) 
                                           ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Aentry binders)),
                                                       (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                          (Extend.Aentry reduce)),
                             (Extend.Aentry lconstr)),
                (fun c red _ bl loc -> 
# 368 "vernac/g_vernac.mlg"
        if List.exists (function CLocalPattern _ -> true | _ -> false) bl
      then
        (* FIXME: "red" will be applied to types in bl and Cast with remain *)
        let c = mkLambdaCN ~loc bl c in
        DefineBody ([], red, c, None)
      else
        (match c with
        | { CAst.v = CCast(c, CastConv t) } -> DefineBody (bl, red, c, Some t)
        | _ -> DefineBody (bl, red, c, None)) 
                                       ))])])
        in let () =
        Pcoq.grammar_extend reduce None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 392 "vernac/g_vernac.mlg"
             None 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Eval"))))),
                                          (Extend.Aentry red_expr)),
                             (Extend.Atoken (Tok.PKEYWORD ("in")))),
                (fun _ r _ loc -> 
# 391 "vernac/g_vernac.mlg"
                                              Some r 
                                  ))])])
        in let () =
        Pcoq.grammar_extend one_decl_notation None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Aentry ne_lstring)),
                                                        (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                           (Extend.Aentry constr)),
                              (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                          (Extend.Atoken (Tok.PIDENT (None)))),
                                                          (fun sc _ loc -> 
                                                          
# 396 "vernac/g_vernac.mlg"
                                           sc 
                                                          ))]))),
                 (fun scopt c _ ntn loc -> 
# 396 "vernac/g_vernac.mlg"
                                                       (ntn,c,scopt) 
                                           ))])])
        in let () =
        Pcoq.grammar_extend decl_sep None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("and"))))),
                 (fun _ loc -> 
# 399 "vernac/g_vernac.mlg"
                         () 
                               ))])])
        in let () =
        Pcoq.grammar_extend decl_notation None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 403 "vernac/g_vernac.mlg"
           [] 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("where")))),
                             (Extend.Alist1sep ((Extend.Aentry one_decl_notation), (Extend.Aentry decl_sep)))),
                (fun l _ loc -> 
# 402 "vernac/g_vernac.mlg"
                                                               l 
                                ))])])
        in let () =
        Pcoq.grammar_extend opt_constructors_or_fields None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 408 "vernac/g_vernac.mlg"
             RecordDecl (None, []) 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD (":=")))),
                             (Extend.Aentry constructor_list_or_record_decl)),
                (fun lc _ loc -> 
# 407 "vernac/g_vernac.mlg"
                                                        lc 
                                 ))])])
        in let () =
        Pcoq.grammar_extend inductive_definition None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Aentry opt_coercion)),
                                                        (Extend.Aentry ident_decl)),
                                                        (Extend.Aentry binders)),
                                                        (Extend.Aopt (Extend.Arules 
                                                        [Extend.Rules 
                                                        (Extend.NextNoRec 
                                                        (Extend.NextNoRec 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                        (Extend.Aentry lconstr)),
                                                        (fun c _ loc -> 
                                                        
# 412 "vernac/g_vernac.mlg"
                                        c 
                                                        ))]))),
                                           (Extend.Aentry opt_constructors_or_fields)),
                              (Extend.Aentry decl_notation)),
                 (fun ntn lc c indpar id oc loc -> 
# 414 "vernac/g_vernac.mlg"
             (((oc,id),indpar,c,lc),ntn) 
                                                   ))])])
        in let () =
        Pcoq.grammar_extend constructor_list_or_record_decl None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 424 "vernac/g_vernac.mlg"
              Constructors [] 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("{")))),
                                          (Extend.Aentry record_fields)),
                             (Extend.Atoken (Tok.PKEYWORD ("}")))),
                (fun _ fs _ loc -> 
# 423 "vernac/g_vernac.mlg"
                                         RecordDecl (None,fs) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Aentry identref)),
                                                       (Extend.Atoken (Tok.PKEYWORD ("{")))),
                                          (Extend.Aentry record_fields)),
                             (Extend.Atoken (Tok.PKEYWORD ("}")))),
                (fun _ fs _ cstr loc -> 
# 422 "vernac/g_vernac.mlg"
            RecordDecl (Some cstr,fs) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry identref)),
                             (Extend.Aentry constructor_type)),
                (fun c id loc -> 
# 420 "vernac/g_vernac.mlg"
                                                  Constructors [ c id ] 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Aentry identref)),
                                                       (Extend.Aentry constructor_type)),
                                          (Extend.Atoken (Tok.PKEYWORD ("|")))),
                             (Extend.Alist0sep ((Extend.Aentry constructor), (Extend.Atoken (Tok.PKEYWORD ("|")))))),
                (fun l _ c id loc -> 
# 419 "vernac/g_vernac.mlg"
            Constructors ((c id)::l) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("|")))),
                             (Extend.Alist1sep ((Extend.Aentry constructor), (Extend.Atoken (Tok.PKEYWORD ("|")))))),
                (fun l _ loc -> 
# 417 "vernac/g_vernac.mlg"
                                                Constructors l 
                                ))])])
        in let () =
        Pcoq.grammar_extend opt_coercion None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 433 "vernac/g_vernac.mlg"
              false 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD (">")))),
                (fun _ loc -> 
# 432 "vernac/g_vernac.mlg"
                 true 
                              ))])])
        in let () =
        Pcoq.grammar_extend rec_definition None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Aentry ident_decl)),
                                                        (Extend.Aentry binders_fixannot)),
                                                        (Extend.Aentry rec_type_cstr)),
                                           (Extend.Aopt (Extend.Arules 
                                           [Extend.Rules (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                                         (Extend.Aentry lconstr)),
                                                         (fun def _ loc -> 
                                                         
# 440 "vernac/g_vernac.mlg"
                                                 def 
                                                         ))]))),
                              (Extend.Aentry decl_notation)),
                 (fun notations body_def rtype bl id_decl loc -> 
# 441 "vernac/g_vernac.mlg"
            let binders, rec_order = bl in
            {fname = fst id_decl; univs = snd id_decl; rec_order; binders; rtype; body_def; notations}
          
                                                                 ))])])
        in let () =
        Pcoq.grammar_extend corec_definition None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Aentry ident_decl)),
                                                        (Extend.Aentry binders)),
                                                        (Extend.Aentry rec_type_cstr)),
                                           (Extend.Aopt (Extend.Arules 
                                           [Extend.Rules (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                                         (Extend.Aentry lconstr)),
                                                         (fun def _ loc -> 
                                                         
# 447 "vernac/g_vernac.mlg"
                                                 def 
                                                         ))]))),
                              (Extend.Aentry decl_notation)),
                 (fun notations body_def rtype binders id_decl loc -> 
# 448 "vernac/g_vernac.mlg"
          {fname = fst id_decl; univs = snd id_decl; rec_order = (); binders; rtype; body_def; notations}
        
                                                                    ))])])
        in let () =
        Pcoq.grammar_extend rec_type_cstr None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 453 "vernac/g_vernac.mlg"
             CAst.make ~loc @@ CHole (None, IntroAnonymous, None) 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD (":")))),
                             (Extend.Aentry lconstr)),
                (fun c _ loc -> 
# 452 "vernac/g_vernac.mlg"
                            c 
                                ))])])
        in let () =
        Pcoq.grammar_extend scheme None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aentry identref)),
                                           (Extend.Atoken (Tok.PKEYWORD (":=")))),
                              (Extend.Aentry scheme_kind)),
                 (fun kind _ id loc -> 
# 458 "vernac/g_vernac.mlg"
                                                     (Some id,kind) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry scheme_kind)),
                (fun kind loc -> 
# 457 "vernac/g_vernac.mlg"
                                (None,kind) 
                                 ))])])
        in let () =
        Pcoq.grammar_extend scheme_kind None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Equality"))))),
                                           (Extend.Atoken (Tok.PKEYWORD ("for")))),
                              (Extend.Aentry smart_global)),
                 (fun ind _ _ loc -> 
# 469 "vernac/g_vernac.mlg"
                                                          EqualityScheme(ind) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Case"))))),
                                                                    (Extend.Atoken (Tok.PKEYWORD ("for")))),
                                                       (Extend.Aentry smart_global)),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Sort"))))),
                             (Extend.Aentry sort_family)),
                (fun s _ ind _ _ loc -> 
# 468 "vernac/g_vernac.mlg"
                                            CaseScheme(false,ind,s) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Elimination"))))),
                                                                    (Extend.Atoken (Tok.PKEYWORD ("for")))),
                                                       (Extend.Aentry smart_global)),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Sort"))))),
                             (Extend.Aentry sort_family)),
                (fun s _ ind _ _ loc -> 
# 466 "vernac/g_vernac.mlg"
                                            CaseScheme(true,ind,s) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Minimality"))))),
                                                                    (Extend.Atoken (Tok.PKEYWORD ("for")))),
                                                       (Extend.Aentry smart_global)),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Sort"))))),
                             (Extend.Aentry sort_family)),
                (fun s _ ind _ _ loc -> 
# 464 "vernac/g_vernac.mlg"
                                            InductionScheme(false,ind,s) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Induction"))))),
                                                                    (Extend.Atoken (Tok.PKEYWORD ("for")))),
                                                       (Extend.Aentry smart_global)),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Sort"))))),
                             (Extend.Aentry sort_family)),
                (fun s _ ind _ _ loc -> 
# 462 "vernac/g_vernac.mlg"
                                            InductionScheme(true,ind,s) 
                                        ))])])
        in let () =
        Pcoq.grammar_extend record_field None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Alist0 (Extend.Aentry quoted_attributes))),
                                                        (Extend.Aentry record_binder)),
                                           (Extend.Aopt (Extend.Arules 
                                           [Extend.Rules (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD ("|")))),
                                                         (Extend.Aentry natural)),
                                                         (fun n _ loc -> 
                                                         
# 486 "vernac/g_vernac.mlg"
                                                                    n 
                                                         ))]))),
                              (Extend.Aentry decl_notation)),
                 (fun rf_notation rf_priority bd attr loc -> 
# 487 "vernac/g_vernac.mlg"
                                      
      let rf_canonical = attr |> List.flatten |> parse canonical in
      let rf_subclass, rf_decl = bd in
      rf_decl, { rf_subclass ; rf_priority ; rf_notation ; rf_canonical } 
                                                             ))])])
        in let () =
        Pcoq.grammar_extend record_fields None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 496 "vernac/g_vernac.mlg"
             [] 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry record_field)),
                (fun f loc -> 
# 495 "vernac/g_vernac.mlg"
                              [f] 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry record_field)),
                             (Extend.Atoken (Tok.PKEYWORD (";")))),
                (fun _ f loc -> 
# 494 "vernac/g_vernac.mlg"
                                   [f] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry record_field)),
                                          (Extend.Atoken (Tok.PKEYWORD (";")))),
                             (Extend.Aentry record_fields)),
                (fun fs _ f loc -> 
# 493 "vernac/g_vernac.mlg"
                                                       f :: fs 
                                   ))])])
        in let () =
        Pcoq.grammar_extend record_binder_body None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aentry binders)),
                                           (Extend.Atoken (Tok.PKEYWORD (":=")))),
                              (Extend.Aentry lconstr)),
                 (fun b _ l loc -> 
# 505 "vernac/g_vernac.mlg"
                                            fun id ->
         match b.CAst.v with
         | CCast(b', (CastConv t|CastVM t|CastNative t)) ->
             (None,DefExpr(id,mkLambdaCN ~loc l b',Some (mkProdCN ~loc l t)))
         | _ ->
             (None,DefExpr(id,mkLambdaCN ~loc l b,None)) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Aentry binders)),
                                                                    (Extend.Aentry of_type_with_opt_coercion)),
                                                       (Extend.Aentry lconstr)),
                                          (Extend.Atoken (Tok.PKEYWORD (":=")))),
                             (Extend.Aentry lconstr)),
                (fun b _ t oc l loc -> 
# 503 "vernac/g_vernac.mlg"
                                             fun id ->
           (oc,DefExpr (id,mkLambdaCN ~loc l b,Some (mkProdCN ~loc l t))) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry binders)),
                                          (Extend.Aentry of_type_with_opt_coercion)),
                             (Extend.Aentry lconstr)),
                (fun t oc l loc -> 
# 501 "vernac/g_vernac.mlg"
                          fun id -> (oc,AssumExpr (id,mkProdCN ~loc l t)) 
                                   ))])])
        in let () =
        Pcoq.grammar_extend record_binder None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry name)),
                              (Extend.Aentry record_binder_body)),
                 (fun f id loc -> 
# 514 "vernac/g_vernac.mlg"
                                               f id 
                                  ));
                Extend.Rule (Extend.Next (Extend.Stop, (Extend.Aentry name)),
                (fun id loc -> 
# 513 "vernac/g_vernac.mlg"
                       (None,AssumExpr(id, CAst.make ~loc @@ CHole (None, IntroAnonymous, None))) 
                               ))])])
        in let () =
        Pcoq.grammar_extend assum_list None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry simple_assum_coe)),
                 (fun b loc -> 
# 517 "vernac/g_vernac.mlg"
                                                                   [b] 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Alist1 (Extend.Aentry assum_coe))),
                (fun bl loc -> 
# 517 "vernac/g_vernac.mlg"
                                  bl 
                               ))])])
        in let () =
        Pcoq.grammar_extend assum_coe None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                           (Extend.Aentry simple_assum_coe)),
                              (Extend.Atoken (Tok.PKEYWORD (")")))),
                 (fun _ a _ loc -> 
# 520 "vernac/g_vernac.mlg"
                                            a 
                                   ))])])
        in let () =
        Pcoq.grammar_extend simple_assum_coe None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Alist1 (Extend.Aentry ident_decl))),
                                           (Extend.Aentry of_type_with_opt_coercion)),
                              (Extend.Aentry lconstr)),
                 (fun c oc idl loc -> 
# 524 "vernac/g_vernac.mlg"
          (not (Option.is_empty oc),(idl,c)) 
                                      ))])])
        in let () =
        Pcoq.grammar_extend constructor_type None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry binders)),
                              (Extend.Arules [Extend.Rules (Extend.Stop, (fun
                                                           loc -> 
# 532 "vernac/g_vernac.mlg"
                   fun l id -> (false,(id,mkProdCN ~loc l (CAst.make ~loc @@ CHole (None, IntroAnonymous, None)))) 
                                                                  ));
                                             Extend.Rules (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Aentry of_type_with_opt_coercion)),
                                                          (Extend.Aentry lconstr)),
                                                          (fun c coe loc ->
                                                          
# 530 "vernac/g_vernac.mlg"
                      fun l id -> (not (Option.is_empty coe),(id,mkProdCN ~loc l c)) 
                                                          ))])),
                 (fun t l loc -> 
# 533 "vernac/g_vernac.mlg"
              t l 
                                 ))])])
        in let () =
        Pcoq.grammar_extend constructor None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry identref)),
                              (Extend.Aentry constructor_type)),
                 (fun c id loc -> 
# 538 "vernac/g_vernac.mlg"
                                               c id 
                                  ))])])
        in let () =
        Pcoq.grammar_extend of_type_with_opt_coercion None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PKEYWORD (":")))),
                 (fun _ loc -> 
# 546 "vernac/g_vernac.mlg"
                   None 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD (":")))),
                             (Extend.Atoken (Tok.PKEYWORD (">")))),
                (fun _ _ loc -> 
# 545 "vernac/g_vernac.mlg"
                        Some true 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD (":")))),
                                          (Extend.Atoken (Tok.PKEYWORD (">")))),
                             (Extend.Atoken (Tok.PKEYWORD (">")))),
                (fun _ _ _ loc -> 
# 544 "vernac/g_vernac.mlg"
                             Some false 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD (":>")))),
                (fun _ loc -> 
# 543 "vernac/g_vernac.mlg"
                    Some true 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD (":>")))),
                             (Extend.Atoken (Tok.PKEYWORD (">")))),
                (fun _ _ loc -> 
# 542 "vernac/g_vernac.mlg"
                         Some false 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD (":>>")))),
                (fun _ loc -> 
# 541 "vernac/g_vernac.mlg"
                   Some false 
                              ))])])
        in ()


# 550 "vernac/g_vernac.mlg"
 

let only_starredidentrefs =
  Pcoq.Entry.of_parser "test_only_starredidentrefs"
    (fun _ strm ->
      let rec aux n =
      match Util.stream_nth n strm with
        | KEYWORD "." -> ()
        | KEYWORD ")" -> ()
        | (IDENT _ | KEYWORD "Type" | KEYWORD "*") -> aux (n+1)
        | _ -> raise Stream.Failure in
      aux 0)
let starredidentreflist_to_expr l =
  match l with
  | [] -> SsEmpty
  | x :: xs -> List.fold_right (fun i acc -> SsUnion(i,acc)) xs x

let warn_deprecated_include_type =
  CWarnings.create ~name:"deprecated-include-type" ~category:"deprecated"
         (fun () -> strbrk "Include Type is deprecated; use Include instead")



let _ = let export_token = Pcoq.Entry.create "export_token"
        and ext_module_type = Pcoq.Entry.create "ext_module_type"
        and ext_module_expr = Pcoq.Entry.create "ext_module_expr"
        and check_module_type = Pcoq.Entry.create "check_module_type"
        and check_module_types = Pcoq.Entry.create "check_module_types"
        and of_module_type = Pcoq.Entry.create "of_module_type"
        and is_module_type = Pcoq.Entry.create "is_module_type"
        and is_module_expr = Pcoq.Entry.create "is_module_expr"
        and functor_app_annot = Pcoq.Entry.create "functor_app_annot"
        and module_expr_inl = Pcoq.Entry.create "module_expr_inl"
        and module_type_inl = Pcoq.Entry.create "module_type_inl"
        and module_binder = Pcoq.Entry.create "module_binder"
        and module_expr_atom = Pcoq.Entry.create "module_expr_atom"
        and with_declaration = Pcoq.Entry.create "with_declaration"
        and starredidentref = Pcoq.Entry.create "starredidentref"
        and ssexpr = Pcoq.Entry.create "ssexpr"
        in
        let () =
        Pcoq.grammar_extend gallina_ext None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Include"))))),
                                                        (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                                           (Extend.Aentry module_type_inl)),
                              (Extend.Alist0 (Extend.Aentry ext_module_type))),
                 (fun l e _ _ loc -> 
# 612 "vernac/g_vernac.mlg"
           warn_deprecated_include_type ~loc ();
        VernacInclude(e::l) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Include"))))),
                                          (Extend.Aentry module_type_inl)),
                             (Extend.Alist0 (Extend.Aentry ext_module_expr))),
                (fun l e _ loc -> 
# 610 "vernac/g_vernac.mlg"
            VernacInclude(e::l) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Export"))))),
                             (Extend.Alist1 (Extend.Aentry global))),
                (fun qidl _ loc -> 
# 608 "vernac/g_vernac.mlg"
                                                 VernacImport (true,qidl) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Import"))))),
                             (Extend.Alist1 (Extend.Aentry global))),
                (fun qidl _ loc -> 
# 607 "vernac/g_vernac.mlg"
                                                 VernacImport (false,qidl) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("From"))))),
                                                                    (Extend.Aentry global)),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Require"))))),
                                          (Extend.Aentry export_token)),
                             (Extend.Alist1 (Extend.Aentry global))),
                (fun qidl export _ ns _ loc -> 
# 606 "vernac/g_vernac.mlg"
          VernacRequire (Some ns, export, qidl) 
                                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Require"))))),
                                          (Extend.Aentry export_token)),
                             (Extend.Alist1 (Extend.Aentry global))),
                (fun qidl export _ loc -> 
# 603 "vernac/g_vernac.mlg"
            VernacRequire (None, export, qidl) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Collection"))))),
                                                       (Extend.Aentry identref)),
                                          (Extend.Atoken (Tok.PKEYWORD (":=")))),
                             (Extend.Aentry section_subset_expr)),
                (fun expr _ id _ loc -> 
# 599 "vernac/g_vernac.mlg"
            VernacNameSectionHypSet (id, expr) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("End"))))),
                             (Extend.Aentry identref)),
                (fun id _ loc -> 
# 595 "vernac/g_vernac.mlg"
                                        VernacEndSegment id 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Chapter"))))),
                             (Extend.Aentry identref)),
                (fun id _ loc -> 
# 592 "vernac/g_vernac.mlg"
                                            VernacBeginSection id 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Section"))))),
                             (Extend.Aentry identref)),
                (fun id _ loc -> 
# 591 "vernac/g_vernac.mlg"
                                            VernacBeginSection id 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Declare"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Module"))))),
                                                                    (Extend.Aentry export_token)),
                                                                    (Extend.Aentry identref)),
                                                       (Extend.Alist0 (Extend.Aentry module_binder))),
                                          (Extend.Atoken (Tok.PKEYWORD (":")))),
                             (Extend.Aentry module_type_inl)),
                (fun mty _ bl id export _ _ loc -> 
# 589 "vernac/g_vernac.mlg"
            VernacDeclareModule (export, id, bl, mty) 
                                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Module"))))),
                                                                    (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                                                                    (Extend.Aentry identref)),
                                                       (Extend.Alist0 (Extend.Aentry module_binder))),
                                          (Extend.Aentry check_module_types)),
                             (Extend.Aentry is_module_type)),
                (fun body sign bl id _ _ loc -> 
# 586 "vernac/g_vernac.mlg"
            VernacDeclareModuleType (id, bl, sign, body) 
                                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Module"))))),
                                                                    (Extend.Aentry export_token)),
                                                                    (Extend.Aentry identref)),
                                                       (Extend.Alist0 (Extend.Aentry module_binder))),
                                          (Extend.Aentry of_module_type)),
                             (Extend.Aentry is_module_expr)),
                (fun body sign bl id export _ loc -> 
# 582 "vernac/g_vernac.mlg"
            VernacDefineModule (export, id, bl, sign, body) 
                                                     ))])])
        in let () =
        Pcoq.grammar_extend export_token None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 618 "vernac/g_vernac.mlg"
              None 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Export"))))),
                (fun _ loc -> 
# 617 "vernac/g_vernac.mlg"
                            Some true 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Import"))))),
                (fun _ loc -> 
# 616 "vernac/g_vernac.mlg"
                            Some false 
                              ))])])
        in let () =
        Pcoq.grammar_extend ext_module_type None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("<+")))),
                              (Extend.Aentry module_type_inl)),
                 (fun mty _ loc -> 
# 621 "vernac/g_vernac.mlg"
                                         mty 
                                   ))])])
        in let () =
        Pcoq.grammar_extend ext_module_expr None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("<+")))),
                              (Extend.Aentry module_expr_inl)),
                 (fun mexpr _ loc -> 
# 624 "vernac/g_vernac.mlg"
                                           mexpr 
                                     ))])])
        in let () =
        Pcoq.grammar_extend check_module_type None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("<:")))),
                              (Extend.Aentry module_type_inl)),
                 (fun mty _ loc -> 
# 627 "vernac/g_vernac.mlg"
                                         mty 
                                   ))])])
        in let () =
        Pcoq.grammar_extend check_module_types None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Alist0 (Extend.Aentry check_module_type))),
                 (fun mtys loc -> 
# 630 "vernac/g_vernac.mlg"
                                            mtys 
                                  ))])])
        in let () =
        Pcoq.grammar_extend of_module_type None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Aentry check_module_types)),
                 (fun mtys loc -> 
# 634 "vernac/g_vernac.mlg"
                                       Check mtys 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD (":")))),
                             (Extend.Aentry module_type_inl)),
                (fun mty _ loc -> 
# 633 "vernac/g_vernac.mlg"
                                        Enforce mty 
                                  ))])])
        in let () =
        Pcoq.grammar_extend is_module_type None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 638 "vernac/g_vernac.mlg"
             [] 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                          (Extend.Aentry module_type_inl)),
                             (Extend.Alist0 (Extend.Aentry ext_module_type))),
                (fun l mty _ loc -> 
# 637 "vernac/g_vernac.mlg"
                                                                     (mty::l) 
                                    ))])])
        in let () =
        Pcoq.grammar_extend is_module_expr None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 642 "vernac/g_vernac.mlg"
             [] 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                          (Extend.Aentry module_expr_inl)),
                             (Extend.Alist0 (Extend.Aentry ext_module_expr))),
                (fun l mexpr _ loc -> 
# 641 "vernac/g_vernac.mlg"
                                                                      (mexpr::l) 
                                      ))])])
        in let () =
        Pcoq.grammar_extend functor_app_annot None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 648 "vernac/g_vernac.mlg"
             DefaultInline 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("[")))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("no"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("inline"))))),
                             (Extend.Atoken (Tok.PKEYWORD ("]")))),
                (fun _ _ _ _ loc -> 
# 647 "vernac/g_vernac.mlg"
                                                  NoInline 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("[")))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("inline"))))),
                                                                    (Extend.Atoken (Tok.PKEYWORD ("at")))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("level"))))),
                                          (Extend.Aentry natural)),
                             (Extend.Atoken (Tok.PKEYWORD ("]")))),
                (fun _ i _ _ _ _ loc -> 
# 646 "vernac/g_vernac.mlg"
          InlineAt i 
                                        ))])])
        in let () =
        Pcoq.grammar_extend module_expr_inl None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry module_expr)),
                              (Extend.Aentry functor_app_annot)),
                 (fun a me loc -> 
# 653 "vernac/g_vernac.mlg"
                                                     (me,a) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("!")))),
                             (Extend.Aentry module_expr)),
                (fun me _ loc -> 
# 652 "vernac/g_vernac.mlg"
                                   (me,NoInline) 
                                 ))])])
        in let () =
        Pcoq.grammar_extend module_type_inl None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry module_type)),
                              (Extend.Aentry functor_app_annot)),
                 (fun a me loc -> 
# 657 "vernac/g_vernac.mlg"
                                                     (me,a) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("!")))),
                             (Extend.Aentry module_type)),
                (fun me _ loc -> 
# 656 "vernac/g_vernac.mlg"
                                   (me,NoInline) 
                                 ))])])
        in let () =
        Pcoq.grammar_extend module_binder None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                        (Extend.Aentry export_token)),
                                                        (Extend.Alist1 (Extend.Aentry identref))),
                                                        (Extend.Atoken (Tok.PKEYWORD (":")))),
                                           (Extend.Aentry module_type_inl)),
                              (Extend.Atoken (Tok.PKEYWORD (")")))),
                 (fun _ mty _ idl export _ loc -> 
# 662 "vernac/g_vernac.mlg"
                                         (export,idl,mty) 
                                                  ))])])
        in let () =
        Pcoq.grammar_extend module_expr None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry module_expr)),
                              (Extend.Aentry module_expr_atom)),
                 (fun me2 me1 loc -> 
# 667 "vernac/g_vernac.mlg"
                                                       CAst.make ~loc @@ CMapply (me1,me2) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry module_expr_atom)),
                (fun me loc -> 
# 666 "vernac/g_vernac.mlg"
                                   me 
                               ))])])
        in let () =
        Pcoq.grammar_extend module_expr_atom None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                           (Extend.Aentry module_expr)),
                              (Extend.Atoken (Tok.PKEYWORD (")")))),
                 (fun _ me _ loc -> 
# 671 "vernac/g_vernac.mlg"
                                                                                            me 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry qualid)),
                (fun qid loc -> 
# 671 "vernac/g_vernac.mlg"
                          CAst.make ~loc @@ CMident qid 
                                ))])])
        in let () =
        Pcoq.grammar_extend with_declaration None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Module"))))),
                                                        (Extend.Aentry fullyqualid)),
                                           (Extend.Atoken (Tok.PKEYWORD (":=")))),
                              (Extend.Aentry qualid)),
                 (fun qid _ fqid _ loc -> 
# 677 "vernac/g_vernac.mlg"
            CWith_Module (fqid,qid) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("Definition")))),
                                                                    (Extend.Aentry fullyqualid)),
                                                       (Extend.Aopt (Extend.Aentry univ_decl))),
                                          (Extend.Atoken (Tok.PKEYWORD (":=")))),
                             (Extend.Aentry Constr.lconstr)),
                (fun c _ udecl fqid _ loc -> 
# 675 "vernac/g_vernac.mlg"
            CWith_Definition (fqid,udecl,c) 
                                             ))])])
        in let () =
        Pcoq.grammar_extend module_type None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aentry module_type)),
                                           (Extend.Atoken (Tok.PKEYWORD ("with")))),
                              (Extend.Aentry with_declaration)),
                 (fun decl _ mty loc -> 
# 686 "vernac/g_vernac.mlg"
            CAst.make ~loc @@ CMwith (mty,decl) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry module_type)),
                             (Extend.Aentry module_expr_atom)),
                (fun me mty loc -> 
# 684 "vernac/g_vernac.mlg"
            CAst.make ~loc @@ CMapply (mty,me) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                          (Extend.Aentry module_type)),
                             (Extend.Atoken (Tok.PKEYWORD (")")))),
                (fun _ mt _ loc -> 
# 682 "vernac/g_vernac.mlg"
                                        mt 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry qualid)),
                (fun qid loc -> 
# 681 "vernac/g_vernac.mlg"
                          CAst.make ~loc @@ CMident qid 
                                ))])])
        in let () =
        Pcoq.grammar_extend section_subset_expr None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry ssexpr)),
                 (fun e loc -> 
# 693 "vernac/g_vernac.mlg"
                        e 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry only_starredidentrefs)),
                             (Extend.Alist0 (Extend.Aentry starredidentref))),
                (fun l _ loc -> 
# 692 "vernac/g_vernac.mlg"
            starredidentreflist_to_expr l 
                                ))])])
        in let () =
        Pcoq.grammar_extend starredidentref None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                              (Extend.Atoken (Tok.PKEYWORD ("*")))),
                 (fun _ _ loc -> 
# 699 "vernac/g_vernac.mlg"
                         SsFwdClose SsType 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                (fun _ loc -> 
# 698 "vernac/g_vernac.mlg"
                    SsType 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry identref)),
                             (Extend.Atoken (Tok.PKEYWORD ("*")))),
                (fun _ i loc -> 
# 697 "vernac/g_vernac.mlg"
                               SsFwdClose(SsSingl i) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry identref)),
                (fun i loc -> 
# 696 "vernac/g_vernac.mlg"
                          SsSingl i 
                              ))])])
        in let () =
        Pcoq.grammar_extend ssexpr None
        (None, [(Some ("35"), None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("-")))),
                              (Extend.Aentry ssexpr)),
                 (fun e _ loc -> 
# 703 "vernac/g_vernac.mlg"
                             SsCompl e 
                                 ))]);
               (Some ("50"), None,
               [Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry ssexpr)),
                                          (Extend.Atoken (Tok.PKEYWORD ("+")))),
                             (Extend.Aentry ssexpr)),
                (fun e2 _ e1 loc -> 
# 706 "vernac/g_vernac.mlg"
                                          SsUnion(e1,e2) 
                                    ));
               Extend.Rule
               (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                      (Extend.Aentry ssexpr)),
                                         (Extend.Atoken (Tok.PKEYWORD ("-")))),
                            (Extend.Aentry ssexpr)),
               (fun e2 _ e1 loc -> 
# 705 "vernac/g_vernac.mlg"
                                          SsSubstr(e1,e2) 
                                   ))]);
               (Some ("0"), None,
               [Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                       (Extend.Aentry ssexpr)),
                                          (Extend.Atoken (Tok.PKEYWORD (")")))),
                             (Extend.Atoken (Tok.PKEYWORD ("*")))),
                (fun _ _ e _ loc -> 
# 714 "vernac/g_vernac.mlg"
                                       SsFwdClose e 
                                    ));
               Extend.Rule
               (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                      (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                         (Extend.Aentry ssexpr)),
                            (Extend.Atoken (Tok.PKEYWORD (")")))),
               (fun _ e _ loc -> 
# 713 "vernac/g_vernac.mlg"
                                 e 
                                 ));
               Extend.Rule
               (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                   (Extend.Stop,
                                                                   (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                                   (Extend.Aentry only_starredidentrefs)),
                                                      (Extend.Alist0 (Extend.Aentry starredidentref))),
                                         (Extend.Atoken (Tok.PKEYWORD (")")))),
                            (Extend.Atoken (Tok.PKEYWORD ("*")))),
               (fun _ _ l _ _ loc -> 
# 712 "vernac/g_vernac.mlg"
            SsFwdClose(starredidentreflist_to_expr l) 
                                     ));
               Extend.Rule
               (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                   (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                      (Extend.Aentry only_starredidentrefs)),
                                         (Extend.Alist0 (Extend.Aentry starredidentref))),
                            (Extend.Atoken (Tok.PKEYWORD (")")))),
               (fun _ l _ _ loc -> 
# 710 "vernac/g_vernac.mlg"
            starredidentreflist_to_expr l 
                                   ));
               Extend.Rule
               (Extend.Next (Extend.Stop, (Extend.Aentry starredidentref)),
               (fun i loc -> 
# 708 "vernac/g_vernac.mlg"
                                 i 
                             ))])])
        in ()

let _ = let arguments_modifier = Pcoq.Entry.create "arguments_modifier"
        and scope = Pcoq.Entry.create "scope"
        and argument_spec = Pcoq.Entry.create "argument_spec"
        and argument_spec_block = Pcoq.Entry.create "argument_spec_block"
        and more_implicits_block = Pcoq.Entry.create "more_implicits_block"
        and strategy_level = Pcoq.Entry.create "strategy_level"
        and instance_name = Pcoq.Entry.create "instance_name"
        and reserv_list = Pcoq.Entry.create "reserv_list"
        and reserv_tuple = Pcoq.Entry.create "reserv_tuple"
        and simple_reserv = Pcoq.Entry.create "simple_reserv"
        in
        let () =
        Pcoq.grammar_extend gallina_ext None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("Generalizable"))))),
                              (Extend.Arules [Extend.Rules (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Arules 
                                                           [Extend.Rules 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Atoken (Tok.PIDENT (Some
                                                           ("Variables"))))),
                                                           (fun _ loc -> 
                                                           
# 801 "vernac/g_vernac.mlg"
                                                              () 
                                                           ));
                                                           Extend.Rules 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Atoken (Tok.PKEYWORD ("Variable")))),
                                                           (fun _ loc -> 
                                                           
# 801 "vernac/g_vernac.mlg"
                                () 
                                                           ))])),
                                                           (Extend.Alist1 (Extend.Aentry identref))),
                                                           (fun idl _ loc ->
                                                           
# 802 "vernac/g_vernac.mlg"
                                            Some idl 
                                                           ));
                                             Extend.Rules (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PIDENT (Some
                                                          ("No"))))),
                                                          (Extend.Atoken (Tok.PIDENT (Some
                                                          ("Variables"))))),
                                                          (fun _ _ loc -> 
                                                          
# 800 "vernac/g_vernac.mlg"
                                                  None 
                                                          ));
                                             Extend.Rules (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PIDENT (Some
                                                          ("All"))))),
                                                          (Extend.Atoken (Tok.PIDENT (Some
                                                          ("Variables"))))),
                                                          (fun _ _ loc -> 
                                                          
# 799 "vernac/g_vernac.mlg"
                                                      Some [] 
                                                          ))])),
                 (fun gen _ loc -> 
# 803 "vernac/g_vernac.mlg"
               VernacGeneralizable gen 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Implicit"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Types"))))),
                             (Extend.Aentry reserv_list)),
                (fun bl _ _ loc -> 
# 795 "vernac/g_vernac.mlg"
            test_plural_form_types loc "Implicit Types" bl;
           VernacReserve bl 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Implicit"))))),
                                          (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                             (Extend.Aentry reserv_list)),
                (fun bl _ _ loc -> 
# 792 "vernac/g_vernac.mlg"
             VernacReserve bl 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Arguments"))))),
                                                                    (Extend.Aentry smart_global)),
                                                       (Extend.Alist0 (Extend.Aentry argument_spec_block))),
                                          (Extend.Aopt (Extend.Arules 
                                          [Extend.Rules (Extend.NextNoRec 
                                                        (Extend.NextNoRec 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD (",")))),
                                                        (Extend.Alist1sep ((Extend.Arules 
                                                        [Extend.Rules 
                                                        (Extend.NextNoRec 
                                                        (Extend.Stop,
                                                        (Extend.Alist0 (Extend.Aentry more_implicits_block))),
                                                        (fun impl loc -> 
                                                        
# 783 "vernac/g_vernac.mlg"
                                                     List.flatten impl 
                                                        ))]), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                        (fun impl _ loc -> 
                                                        
# 784 "vernac/g_vernac.mlg"
                         impl 
                                                        ))]))),
                             (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                         (Extend.Alist1sep ((Extend.Aentry arguments_modifier), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                         (fun l _ loc -> 
                                                         
# 786 "vernac/g_vernac.mlg"
                                                                    l 
                                                         ))]))),
                (fun mods more_implicits args qid _ loc -> 
# 787 "vernac/g_vernac.mlg"
           let mods = match mods with None -> [] | Some l -> List.flatten l in
         let more_implicits = Option.default [] more_implicits in
         VernacArguments (qid, List.flatten args, more_implicits, mods) 
                                                           ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Existing"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Class"))))),
                             (Extend.Aentry global)),
                (fun is _ _ loc -> 
# 776 "vernac/g_vernac.mlg"
                                                          VernacExistingClass is 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Existing"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Instances"))))),
                                          (Extend.Alist1 (Extend.Aentry global))),
                             (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD ("|")))),
                                                         (Extend.Aentry natural)),
                                                         (fun i _ loc -> 
                                                         
# 771 "vernac/g_vernac.mlg"
                                          i 
                                                         ))]))),
                (fun pri ids _ _ loc -> 
# 772 "vernac/g_vernac.mlg"
           let info = { Typeclasses.hint_priority = pri; hint_pattern = None } in
         let insts = List.map (fun i -> (i, info)) ids in
          VernacExistingInstance insts 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Existing"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Instance"))))),
                                          (Extend.Aentry global)),
                             (Extend.Aentry hint_info)),
                (fun info id _ _ loc -> 
# 768 "vernac/g_vernac.mlg"
            VernacExistingInstance [id, info] 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Instance"))))),
                                                                    (Extend.Aentry instance_name)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                       (Extend.Aentryl (operconstr, "200"))),
                                          (Extend.Aentry hint_info)),
                             (Extend.Arules [Extend.Rules (Extend.Stop, (fun
                                                          loc -> 
# 763 "vernac/g_vernac.mlg"
                                                            None 
                                                                 ));
                                            Extend.Rules (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                                         (Extend.Aentry lconstr)),
                                                         (fun c _ loc -> 
                                                         
# 763 "vernac/g_vernac.mlg"
                                    Some (false,c) 
                                                         ));
                                            Extend.Rules (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                                         (Extend.Atoken (Tok.PKEYWORD ("{")))),
                                                         (Extend.Aentry record_declaration)),
                                                         (Extend.Atoken (Tok.PKEYWORD ("}")))),
                                                         (fun _ r _ _ loc ->
                                                         
# 762 "vernac/g_vernac.mlg"
                                                               Some (true,r) 
                                                         ))])),
                (fun props info t _ namesup _ loc -> 
# 764 "vernac/g_vernac.mlg"
             VernacInstance (fst namesup,snd namesup,t,props,info) 
                                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Context"))))),
                             (Extend.Alist1 (Extend.Aentry binder))),
                (fun c _ loc -> 
# 757 "vernac/g_vernac.mlg"
            VernacContext (List.flatten c) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Coercion"))))),
                                                                    (Extend.Aentry by_notation)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                       (Extend.Aentry class_rawexpr)),
                                          (Extend.Atoken (Tok.PKEYWORD (">->")))),
                             (Extend.Aentry class_rawexpr)),
                (fun t _ s _ ntn _ loc -> 
# 754 "vernac/g_vernac.mlg"
            VernacCoercion (CAst.make ~loc @@ ByNotation ntn, s, t) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Coercion"))))),
                                                                    (Extend.Aentry global)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                       (Extend.Aentry class_rawexpr)),
                                          (Extend.Atoken (Tok.PKEYWORD (">->")))),
                             (Extend.Aentry class_rawexpr)),
                (fun t _ s _ qid _ loc -> 
# 751 "vernac/g_vernac.mlg"
            VernacCoercion (CAst.make ~loc @@ AN qid, s, t) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Identity"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Coercion"))))),
                                                                    (Extend.Aentry identref)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                       (Extend.Aentry class_rawexpr)),
                                          (Extend.Atoken (Tok.PKEYWORD (">->")))),
                             (Extend.Aentry class_rawexpr)),
                (fun t _ s _ f _ _ loc -> 
# 748 "vernac/g_vernac.mlg"
             VernacIdentityCoercion (f, s, t) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Coercion"))))),
                                                       (Extend.Aentry global)),
                                          (Extend.Aopt (Extend.Aentry univ_decl))),
                             (Extend.Aentry def_body)),
                (fun d u qid _ loc -> 
# 744 "vernac/g_vernac.mlg"
            let s = coerce_reference_to_id qid in
          VernacDefinition ((NoDischarge,Coercion),((CAst.make (Name s)),u),d) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Canonical"))))),
                                          (Extend.Aopt (Extend.Arules 
                                          [Extend.Rules (Extend.NextNoRec 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Structure"))))),
                                                        (fun _ loc -> 
                                                        
# 739 "vernac/g_vernac.mlg"
                                                       ()
                                                        ))]))),
                             (Extend.Aentry by_notation)),
                (fun ntn _ _ loc -> 
# 740 "vernac/g_vernac.mlg"
            VernacCanonical CAst.(make ~loc @@ ByNotation ntn) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Canonical"))))),
                                                       (Extend.Aopt (Extend.Arules 
                                                       [Extend.Rules 
                                                       (Extend.NextNoRec 
                                                       (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Structure"))))),
                                                       (fun _ loc -> 
                                                       
# 732 "vernac/g_vernac.mlg"
                                                       ()
                                                       ))]))),
                                          (Extend.Aentry global)),
                             (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Aopt (Extend.Aentry univ_decl))),
                                                         (Extend.Aentry def_body)),
                                                         (fun d u loc -> 
                                                         
# 732 "vernac/g_vernac.mlg"
                                                                                                                            (u,d) 
                                                         ))]))),
                (fun ud qid _ _ loc -> 
# 733 "vernac/g_vernac.mlg"
            match ud with
           | None ->
             VernacCanonical CAst.(make ~loc @@ AN qid)
           | Some (u,d) ->
             let s = coerce_reference_to_id qid in
             VernacDefinition ((NoDischarge,CanonicalStructure),((CAst.make (Name s)),u),d) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Strategy"))))),
                             (Extend.Alist1 (Extend.Arules [Extend.Rules 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.NextNoRec 
                                                           (Extend.Stop,
                                                           (Extend.Aentry strategy_level)),
                                                           (Extend.Atoken (Tok.PKEYWORD ("[")))),
                                                           (Extend.Alist1 (Extend.Aentry smart_global))),
                                                           (Extend.Atoken (Tok.PKEYWORD ("]")))),
                                                           (fun _ q _ v
                                                           loc -> 
# 729 "vernac/g_vernac.mlg"
                                                                        (v,q) 
                                                                  ))]))),
                (fun l _ loc -> 
# 730 "vernac/g_vernac.mlg"
              VernacSetStrategy l 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Opaque"))))),
                             (Extend.Alist1 (Extend.Aentry smart_global))),
                (fun l _ loc -> 
# 727 "vernac/g_vernac.mlg"
            VernacSetOpacity (Conv_oracle.Opaque, l) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Transparent"))))),
                             (Extend.Alist1 (Extend.Aentry smart_global))),
                (fun l _ loc -> 
# 725 "vernac/g_vernac.mlg"
            VernacSetOpacity (Conv_oracle.transparent, l) 
                                ))])])
        in let () =
        Pcoq.grammar_extend arguments_modifier None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("clear"))))),
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("implicits"))))),
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("and"))))),
                              (Extend.Atoken (Tok.PIDENT (Some ("scopes"))))),
                 (fun _ _ _ _ loc -> 
# 818 "vernac/g_vernac.mlg"
            [`ClearImplicits; `ClearScopes] 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("clear"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("scopes"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("and"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("implicits"))))),
                (fun _ _ _ _ loc -> 
# 816 "vernac/g_vernac.mlg"
            [`ClearImplicits; `ClearScopes] 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("extra"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("scopes"))))),
                (fun _ _ loc -> 
# 814 "vernac/g_vernac.mlg"
                                           [`ExtraScopes] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("assert"))))),
                (fun _ loc -> 
# 813 "vernac/g_vernac.mlg"
                            [`Assert] 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("rename"))))),
                (fun _ loc -> 
# 812 "vernac/g_vernac.mlg"
                            [`Rename] 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("clear"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("bidirectionality"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("hint"))))),
                (fun _ _ _ loc -> 
# 811 "vernac/g_vernac.mlg"
                                                                   [`ClearBidiHint] 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("clear"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("scopes"))))),
                (fun _ _ loc -> 
# 810 "vernac/g_vernac.mlg"
                                           [`ClearScopes] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("clear"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("implicits"))))),
                (fun _ _ loc -> 
# 809 "vernac/g_vernac.mlg"
                                              [`ClearImplicits] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("default"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("implicits"))))),
                (fun _ _ loc -> 
# 808 "vernac/g_vernac.mlg"
                                                [`DefaultImplicits] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("simpl"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("never"))))),
                (fun _ _ loc -> 
# 807 "vernac/g_vernac.mlg"
                                          [`ReductionNeverUnfold] 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("simpl"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("nomatch"))))),
                (fun _ _ loc -> 
# 806 "vernac/g_vernac.mlg"
                                            [`ReductionDontExposeCase] 
                                ))])])
        in let () =
        Pcoq.grammar_extend scope None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("%")))),
                              (Extend.Atoken (Tok.PIDENT (None)))),
                 (fun key _ loc -> 
# 822 "vernac/g_vernac.mlg"
                              key 
                                   ))])])
        in let () =
        Pcoq.grammar_extend argument_spec None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Aopt (Extend.Atoken (Tok.PKEYWORD ("!"))))),
                                           (Extend.Aentry name)),
                              (Extend.Aopt (Extend.Aentry scope))),
                 (fun s id b loc -> 
# 826 "vernac/g_vernac.mlg"
         id.CAst.v, not (Option.is_empty b), Option.map (fun x -> CAst.make ~loc x) s 
                                    ))])])
        in let () =
        Pcoq.grammar_extend argument_spec_block None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("{")))),
                                                        (Extend.Alist1 (Extend.Aentry argument_spec))),
                                           (Extend.Atoken (Tok.PKEYWORD ("}")))),
                              (Extend.Aopt (Extend.Aentry scope))),
                 (fun sc _ items _ loc -> 
# 855 "vernac/g_vernac.mlg"
         let f x = match sc, x with
         | None, x -> x | x, None -> Option.map (fun y -> CAst.make ~loc y) x
         | Some _, Some _ -> user_err Pp.(str "scope declared twice") in
       List.map (fun (name,recarg_like,notation_scope) ->
           RealArg { name=name; recarg_like=recarg_like;
                 notation_scope=f notation_scope;
                 implicit_status = MaximallyImplicit}) items 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("[")))),
                                                       (Extend.Alist1 (Extend.Aentry argument_spec))),
                                          (Extend.Atoken (Tok.PKEYWORD ("]")))),
                             (Extend.Aopt (Extend.Aentry scope))),
                (fun sc _ items _ loc -> 
# 847 "vernac/g_vernac.mlg"
         let f x = match sc, x with
         | None, x -> x | x, None -> Option.map (fun y -> CAst.make ~loc y) x
         | Some _, Some _ -> user_err Pp.(str "scope declared twice") in
       List.map (fun (name,recarg_like,notation_scope) ->
           RealArg { name=name; recarg_like=recarg_like;
                 notation_scope=f notation_scope;
                 implicit_status = Implicit}) items 
                                         ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                       (Extend.Alist1 (Extend.Aentry argument_spec))),
                                          (Extend.Atoken (Tok.PKEYWORD (")")))),
                             (Extend.Aopt (Extend.Aentry scope))),
                (fun sc _ items _ loc -> 
# 839 "vernac/g_vernac.mlg"
         let f x = match sc, x with
         | None, x -> x | x, None -> Option.map (fun y -> CAst.make ~loc y) x
         | Some _, Some _ -> user_err Pp.(str "scope declared twice") in
       List.map (fun (name,recarg_like,notation_scope) ->
           RealArg { name=name; recarg_like=recarg_like;
                 notation_scope=f notation_scope;
                 implicit_status = NotImplicit}) items 
                                         ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("&")))),
                (fun _ loc -> 
# 837 "vernac/g_vernac.mlg"
               [BidiArg] 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("/")))),
                (fun _ loc -> 
# 836 "vernac/g_vernac.mlg"
               [VolatileArg] 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry argument_spec)),
                (fun item loc -> 
# 832 "vernac/g_vernac.mlg"
        let name, recarg_like, notation_scope = item in
      [RealArg { name=name; recarg_like=recarg_like;
             notation_scope=notation_scope;
             implicit_status = NotImplicit}] 
                                 ))])])
        in let () =
        Pcoq.grammar_extend more_implicits_block None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("{")))),
                                           (Extend.Alist1 (Extend.Aentry name))),
                              (Extend.Atoken (Tok.PKEYWORD ("}")))),
                 (fun _ items _ loc -> 
# 870 "vernac/g_vernac.mlg"
         List.map (fun name -> (name.CAst.v, MaximallyImplicit)) items 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("[")))),
                                          (Extend.Alist1 (Extend.Aentry name))),
                             (Extend.Atoken (Tok.PKEYWORD ("]")))),
                (fun _ items _ loc -> 
# 868 "vernac/g_vernac.mlg"
         List.map (fun name -> (name.CAst.v, Impargs.Implicit)) items 
                                      ));
                Extend.Rule (Extend.Next (Extend.Stop, (Extend.Aentry name)),
                (fun name loc -> 
# 866 "vernac/g_vernac.mlg"
                       [(name.CAst.v, NotImplicit)] 
                                 ))])])
        in let () =
        Pcoq.grammar_extend strategy_level None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("transparent"))))),
                 (fun _ loc -> 
# 877 "vernac/g_vernac.mlg"
                                 Conv_oracle.transparent 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry integer)),
                (fun n loc -> 
# 876 "vernac/g_vernac.mlg"
                       Conv_oracle.Level n 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("opaque"))))),
                (fun _ loc -> 
# 875 "vernac/g_vernac.mlg"
                            Conv_oracle.Opaque 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("expand"))))),
                (fun _ loc -> 
# 874 "vernac/g_vernac.mlg"
                            Conv_oracle.Expand 
                              ))])])
        in let () =
        Pcoq.grammar_extend instance_name None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 882 "vernac/g_vernac.mlg"
             ((CAst.make ~loc Anonymous), None), []  
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry ident_decl)),
                             (Extend.Aentry binders)),
                (fun bl name loc -> 
# 881 "vernac/g_vernac.mlg"
            (CAst.map (fun id -> Name id) (fst name), snd name), bl 
                                    ))])])
        in let () =
        Pcoq.grammar_extend hint_info None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 887 "vernac/g_vernac.mlg"
             { Typeclasses.hint_priority = None; hint_pattern = None } 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("|")))),
                                          (Extend.Aopt (Extend.Aentry natural))),
                             (Extend.Aopt (Extend.Aentry constr_pattern))),
                (fun pat i _ loc -> 
# 886 "vernac/g_vernac.mlg"
           { Typeclasses.hint_priority = i; hint_pattern = pat } 
                                    ))])])
        in let () =
        Pcoq.grammar_extend reserv_list None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry simple_reserv)),
                 (fun b loc -> 
# 890 "vernac/g_vernac.mlg"
                                                                   [b] 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Alist1 (Extend.Aentry reserv_tuple))),
                (fun bl loc -> 
# 890 "vernac/g_vernac.mlg"
                                     bl 
                               ))])])
        in let () =
        Pcoq.grammar_extend reserv_tuple None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                           (Extend.Aentry simple_reserv)),
                              (Extend.Atoken (Tok.PKEYWORD (")")))),
                 (fun _ a _ loc -> 
# 893 "vernac/g_vernac.mlg"
                                         a 
                                   ))])])
        in let () =
        Pcoq.grammar_extend simple_reserv None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Alist1 (Extend.Aentry identref))),
                                           (Extend.Atoken (Tok.PKEYWORD (":")))),
                              (Extend.Aentry lconstr)),
                 (fun c _ idl loc -> 
# 896 "vernac/g_vernac.mlg"
                                                    (idl,c) 
                                     ))])])
        in ()

let _ = let printable = Pcoq.Entry.create "printable"
        and printunivs_subgraph = Pcoq.Entry.create "printunivs_subgraph"
        and locatable = Pcoq.Entry.create "locatable"
        and option_setting = Pcoq.Entry.create "option_setting"
        and option_ref_value = Pcoq.Entry.create "option_ref_value"
        and option_table = Pcoq.Entry.create "option_table"
        and as_dirpath = Pcoq.Entry.create "as_dirpath"
        and ne_in_or_out_modules = Pcoq.Entry.create "ne_in_or_out_modules"
        and in_or_out_modules = Pcoq.Entry.create "in_or_out_modules"
        and comment = Pcoq.Entry.create "comment"
        and positive_search_mark = Pcoq.Entry.create "positive_search_mark"
        and scope = Pcoq.Entry.create "scope"
        and searchabout_query = Pcoq.Entry.create "searchabout_query"
        and searchabout_queries = Pcoq.Entry.create "searchabout_queries"
        and univ_name_list = Pcoq.Entry.create "univ_name_list"
        in
        let () =
        Pcoq.grammar_extend gallina_ext None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Export"))))),
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("Unset"))))),
                              (Extend.Aentry option_table)),
                 (fun table _ _ loc -> 
# 908 "vernac/g_vernac.mlg"
            VernacSetOption (true, table, OptionUnset) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Export"))))),
                                                       (Extend.Atoken (Tok.PKEYWORD ("Set")))),
                                          (Extend.Aentry option_table)),
                             (Extend.Aentry option_setting)),
                (fun v table _ _ loc -> 
# 906 "vernac/g_vernac.mlg"
          VernacSetOption (true, table, v) 
                                        ))])])
        in let () =
        Pcoq.grammar_extend command None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Remove"))))),
                                           (Extend.Atoken (Tok.PIDENT (None)))),
                              (Extend.Alist1 (Extend.Aentry option_ref_value))),
                 (fun v table _ loc -> 
# 990 "vernac/g_vernac.mlg"
            VernacRemoveOption ([table], v) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Remove"))))),
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                          (Extend.Atoken (Tok.PIDENT (None)))),
                             (Extend.Alist1 (Extend.Aentry option_ref_value))),
                (fun v field table _ loc -> 
# 988 "vernac/g_vernac.mlg"
             VernacRemoveOption ([table;field], v) 
                                            ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Test"))))),
                             (Extend.Aentry option_table)),
                (fun table _ loc -> 
# 985 "vernac/g_vernac.mlg"
            VernacPrintOption table 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Test"))))),
                                                       (Extend.Aentry option_table)),
                                          (Extend.Atoken (Tok.PKEYWORD ("for")))),
                             (Extend.Alist1 (Extend.Aentry option_ref_value))),
                (fun v _ table _ loc -> 
# 983 "vernac/g_vernac.mlg"
             VernacMemOption (table, v) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Add"))))),
                                          (Extend.Atoken (Tok.PIDENT (None)))),
                             (Extend.Alist1 (Extend.Aentry option_ref_value))),
                (fun v table _ loc -> 
# 980 "vernac/g_vernac.mlg"
            VernacAddOption ([table], v) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Add"))))),
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                          (Extend.Atoken (Tok.PIDENT (None)))),
                             (Extend.Alist1 (Extend.Aentry option_ref_value))),
                (fun v field table _ loc -> 
# 974 "vernac/g_vernac.mlg"
             VernacAddOption ([table;field], v) 
                                            ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Print"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Table"))))),
                             (Extend.Aentry option_table)),
                (fun table _ _ loc -> 
# 971 "vernac/g_vernac.mlg"
            VernacPrintOption table 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Unset"))))),
                             (Extend.Aentry option_table)),
                (fun table _ loc -> 
# 968 "vernac/g_vernac.mlg"
            VernacSetOption (false, table, OptionUnset) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("Set")))),
                                          (Extend.Aentry option_table)),
                             (Extend.Aentry option_setting)),
                (fun v table _ loc -> 
# 966 "vernac/g_vernac.mlg"
            VernacSetOption (false, table, v) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Add"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Rec"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("ML"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Path"))))),
                             (Extend.Aentry ne_string)),
                (fun dir _ _ _ _ loc -> 
# 962 "vernac/g_vernac.mlg"
            VernacAddMLPath (true, dir) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Add"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("ML"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Path"))))),
                             (Extend.Aentry ne_string)),
                (fun dir _ _ _ loc -> 
# 960 "vernac/g_vernac.mlg"
            VernacAddMLPath (false, dir) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Inspect"))))),
                             (Extend.Aentry natural)),
                (fun n _ loc -> 
# 957 "vernac/g_vernac.mlg"
                                          VernacPrint (PrintInspect n) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Print"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Namespace"))))),
                             (Extend.Aentry dirpath)),
                (fun ns _ _ loc -> 
# 956 "vernac/g_vernac.mlg"
            VernacPrint (PrintNamespace ns) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Print"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Module"))))),
                             (Extend.Aentry global)),
                (fun qid _ _ loc -> 
# 954 "vernac/g_vernac.mlg"
            VernacPrint (PrintModule qid) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Print"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Module"))))),
                                          (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                             (Extend.Aentry global)),
                (fun qid _ _ _ loc -> 
# 952 "vernac/g_vernac.mlg"
            VernacPrint (PrintModuleType qid) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Print"))))),
                                          (Extend.Aentry smart_global)),
                             (Extend.Aopt (Extend.Aentry univ_name_list))),
                (fun l qid _ loc -> 
# 950 "vernac/g_vernac.mlg"
                                                                       VernacPrint (PrintName (qid,l)) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Print"))))),
                             (Extend.Aentry printable)),
                (fun p _ loc -> 
# 949 "vernac/g_vernac.mlg"
                                          VernacPrint p 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("Type")))),
                             (Extend.Aentry lconstr)),
                (fun c _ loc -> 
# 946 "vernac/g_vernac.mlg"
                                 VernacGlobalCheck c 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Remove"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("LoadPath"))))),
                             (Extend.Aentry ne_string)),
                (fun dir _ _ loc -> 
# 943 "vernac/g_vernac.mlg"
            VernacRemoveLoadPath dir 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Add"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Rec"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("LoadPath"))))),
                                          (Extend.Aentry ne_string)),
                             (Extend.Aentry as_dirpath)),
                (fun alias dir _ _ _ loc -> 
# 941 "vernac/g_vernac.mlg"
                                  VernacAddLoadPath (true, dir, alias) 
                                            ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Add"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("LoadPath"))))),
                                          (Extend.Aentry ne_string)),
                             (Extend.Aentry as_dirpath)),
                (fun alias dir _ _ loc -> 
# 939 "vernac/g_vernac.mlg"
            VernacAddLoadPath (false, dir, alias) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Locate"))))),
                             (Extend.Aentry locatable)),
                (fun l _ loc -> 
# 935 "vernac/g_vernac.mlg"
                                           VernacLocate l 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Declare"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("ML"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Module"))))),
                             (Extend.Alist1 (Extend.Aentry ne_string))),
                (fun l _ _ _ loc -> 
# 933 "vernac/g_vernac.mlg"
            VernacDeclareMLModule l 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Load"))))),
                                          (Extend.Arules [Extend.Rules 
                                                         (Extend.Stop, (fun
                                                         loc -> 
# 929 "vernac/g_vernac.mlg"
                                                                       false 
                                                                ));
                                                         Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PIDENT (Some
                                                         ("Verbose"))))),
                                                         (fun _ loc -> 
                                                         
# 929 "vernac/g_vernac.mlg"
                                                         true 
                                                         ))])),
                             (Extend.Arules [Extend.Rules (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PIDENT (None)))),
                                                          (fun s loc -> 
                                                          
# 930 "vernac/g_vernac.mlg"
                                                      s 
                                                          ));
                                            Extend.Rules (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Aentry ne_string)),
                                                         (fun s loc -> 
                                                         
# 930 "vernac/g_vernac.mlg"
                                 s 
                                                         ))])),
                (fun s verbosely _ loc -> 
# 931 "vernac/g_vernac.mlg"
            VernacLoad (verbosely, s) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Cd"))))),
                             (Extend.Aentry ne_string)),
                (fun dir _ loc -> 
# 927 "vernac/g_vernac.mlg"
                                         VernacChdir (Some dir) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Cd"))))),
                (fun _ loc -> 
# 926 "vernac/g_vernac.mlg"
                        VernacChdir None 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Pwd"))))),
                (fun _ loc -> 
# 925 "vernac/g_vernac.mlg"
                         VernacChdir None 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Declare"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Scope"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun sc _ _ loc -> 
# 922 "vernac/g_vernac.mlg"
            VernacDeclareScope sc 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Declare"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Instance"))))),
                                                                    (Extend.Aentry ident_decl)),
                                                                    (Extend.Aentry binders)),
                                                       (Extend.Atoken (Tok.PKEYWORD (":")))),
                                          (Extend.Aentryl (operconstr, "200"))),
                             (Extend.Aentry hint_info)),
                (fun info t _ bl id _ _ loc -> 
# 918 "vernac/g_vernac.mlg"
             VernacDeclareInstance (id, bl, t, info) 
                                               ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Comments"))))),
                             (Extend.Alist0 (Extend.Aentry comment))),
                (fun l _ loc -> 
# 912 "vernac/g_vernac.mlg"
                                                 VernacComments l 
                                ))])])
        in let () =
        Pcoq.grammar_extend query_command None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("SearchAbout"))))),
                                                        (Extend.Atoken (Tok.PKEYWORD ("[")))),
                                                        (Extend.Alist1 (Extend.Aentry searchabout_query))),
                                                        (Extend.Atoken (Tok.PKEYWORD ("]")))),
                                           (Extend.Aentry in_or_out_modules)),
                              (Extend.Atoken (Tok.PKEYWORD (".")))),
                 (fun _ l _ sl _ _ loc -> 
# 1016 "vernac/g_vernac.mlg"
           fun g -> VernacSearch (SearchAbout sl,g, l) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("SearchAbout"))))),
                                                       (Extend.Aentry searchabout_query)),
                                          (Extend.Aentry searchabout_queries)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ l s _ loc -> 
# 1012 "vernac/g_vernac.mlg"
            fun g -> let (sl,m) = l in VernacSearch (SearchAbout (s::sl),g, m) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Search"))))),
                                                       (Extend.Aentry searchabout_query)),
                                          (Extend.Aentry searchabout_queries)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ l s _ loc -> 
# 1009 "vernac/g_vernac.mlg"
            let (sl,m) = l in fun g -> VernacSearch (Search (s::sl),g, m) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("SearchRewrite"))))),
                                                       (Extend.Aentry constr_pattern)),
                                          (Extend.Aentry in_or_out_modules)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ l c _ loc -> 
# 1007 "vernac/g_vernac.mlg"
            fun g -> VernacSearch (SearchRewrite c,g, l) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("SearchPattern"))))),
                                                       (Extend.Aentry constr_pattern)),
                                          (Extend.Aentry in_or_out_modules)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ l c _ loc -> 
# 1005 "vernac/g_vernac.mlg"
            fun g -> VernacSearch (SearchPattern c,g, l) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("SearchHead"))))),
                                                       (Extend.Aentry constr_pattern)),
                                          (Extend.Aentry in_or_out_modules)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ l c _ loc -> 
# 1003 "vernac/g_vernac.mlg"
            fun g -> VernacSearch (SearchHead c,g, l) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("About"))))),
                                                       (Extend.Aentry smart_global)),
                                          (Extend.Aopt (Extend.Aentry univ_name_list))),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ l qid _ loc -> 
# 1001 "vernac/g_vernac.mlg"
           fun g -> VernacPrint (PrintAbout (qid,l,g)) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Check"))))),
                                          (Extend.Aentry lconstr)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ c _ loc -> 
# 998 "vernac/g_vernac.mlg"
           fun g -> VernacCheckMayEval (None, g, c) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Compute"))))),
                                          (Extend.Aentry lconstr)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ c _ loc -> 
# 996 "vernac/g_vernac.mlg"
            fun g -> VernacCheckMayEval (Some (Genredexpr.CbvVm None), g, c) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Eval"))))),
                                                                    (Extend.Aentry red_expr)),
                                                       (Extend.Atoken (Tok.PKEYWORD ("in")))),
                                          (Extend.Aentry lconstr)),
                             (Extend.Atoken (Tok.PKEYWORD (".")))),
                (fun _ c _ r _ loc -> 
# 994 "vernac/g_vernac.mlg"
            fun g -> VernacCheckMayEval (Some r, g, c) 
                                      ))])])
        in let () =
        Pcoq.grammar_extend printable None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                              ("Registered"))))),
                 (fun _ loc -> 
# 1065 "vernac/g_vernac.mlg"
                                PrintRegistered 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Strategies"))))),
                (fun _ loc -> 
# 1064 "vernac/g_vernac.mlg"
                                PrintStrategy None 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Strategy"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ loc -> 
# 1063 "vernac/g_vernac.mlg"
                                                  PrintStrategy (Some qid) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("All"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Dependencies"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ _ loc -> 
# 1062 "vernac/g_vernac.mlg"
                                                                   PrintAssumptions (true, true, qid) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Transparent"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Dependencies"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ _ loc -> 
# 1061 "vernac/g_vernac.mlg"
                                                                           PrintAssumptions (false, true, qid) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Opaque"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Dependencies"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ _ loc -> 
# 1060 "vernac/g_vernac.mlg"
                                                                      PrintAssumptions (true, false, qid) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Assumptions"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ loc -> 
# 1059 "vernac/g_vernac.mlg"
                                                     PrintAssumptions (false, false, qid) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Arules 
                                                                    [Extend.Rules 
                                                                    (Extend.Stop,
                                                                    (fun
                                                                    loc -> 
                                                                    
# 1056 "vernac/g_vernac.mlg"
                                                false 
                                                                    ));
                                                                    Extend.Rules 
                                                                    (Extend.NextNoRec 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Sorted"))))),
                                                                    (fun _
                                                                    loc -> 
                                                                    
# 1056 "vernac/g_vernac.mlg"
                                  true 
                                                                    ))])),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Universes"))))),
                                          (Extend.Aopt (Extend.Aentry printunivs_subgraph))),
                             (Extend.Aopt (Extend.Aentry ne_string))),
                (fun fopt g _ b loc -> 
# 1058 "vernac/g_vernac.mlg"
          PrintUniverses (b, g, fopt) 
                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Implicit"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ loc -> 
# 1055 "vernac/g_vernac.mlg"
                                                  PrintImplicit qid 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Visibility"))))),
                             (Extend.Aopt (Extend.Atoken (Tok.PIDENT (None))))),
                (fun s _ loc -> 
# 1054 "vernac/g_vernac.mlg"
                                               PrintVisibility s 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Scope"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun s _ loc -> 
# 1053 "vernac/g_vernac.mlg"
                                      PrintScope s 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Scopes"))))),
                (fun _ loc -> 
# 1052 "vernac/g_vernac.mlg"
                            PrintScopes 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("HintDb"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun s _ loc -> 
# 1051 "vernac/g_vernac.mlg"
                                       PrintHintDbName s 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Hint"))))),
                             (Extend.Atoken (Tok.PKEYWORD ("*")))),
                (fun _ _ loc -> 
# 1050 "vernac/g_vernac.mlg"
                               PrintHintDb 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Hint"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ loc -> 
# 1049 "vernac/g_vernac.mlg"
                                              PrintHint qid 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Hint"))))),
                (fun _ loc -> 
# 1048 "vernac/g_vernac.mlg"
                          PrintHintGoal 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Options"))))),
                (fun _ loc -> 
# 1047 "vernac/g_vernac.mlg"
                             PrintTables (* A Synonymous to Tables *) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Tables"))))),
                (fun _ loc -> 
# 1046 "vernac/g_vernac.mlg"
                            PrintTables 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Typing"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("Flags"))))),
                (fun _ _ loc -> 
# 1045 "vernac/g_vernac.mlg"
                                           PrintTypingFlags 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Canonical"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("Projections"))))),
                (fun _ _ loc -> 
# 1044 "vernac/g_vernac.mlg"
                                                    PrintCanonicalConversions 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Coercion"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Paths"))))),
                                          (Extend.Aentry class_rawexpr)),
                             (Extend.Aentry class_rawexpr)),
                (fun t s _ _ loc -> 
# 1043 "vernac/g_vernac.mlg"
              PrintCoercionPaths (s,t) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Coercions"))))),
                (fun _ loc -> 
# 1041 "vernac/g_vernac.mlg"
                               PrintCoercions 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Instances"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ loc -> 
# 1040 "vernac/g_vernac.mlg"
                                                   PrintInstances qid 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("TypeClasses"))))),
                (fun _ loc -> 
# 1039 "vernac/g_vernac.mlg"
                                 PrintTypeClasses 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Classes"))))),
                (fun _ loc -> 
# 1038 "vernac/g_vernac.mlg"
                              PrintClasses 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Graph"))))),
                (fun _ loc -> 
# 1037 "vernac/g_vernac.mlg"
                           PrintGraph 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Debug"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("GC"))))),
                (fun _ _ loc -> 
# 1036 "vernac/g_vernac.mlg"
                                       PrintDebugGC 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("ML"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("Modules"))))),
                (fun _ _ loc -> 
# 1035 "vernac/g_vernac.mlg"
                                         PrintMLModules 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("ML"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("Path"))))),
                (fun _ _ loc -> 
# 1034 "vernac/g_vernac.mlg"
                                      PrintMLLoadPath 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Libraries"))))),
                (fun _ loc -> 
# 1032 "vernac/g_vernac.mlg"
                               PrintModules 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Modules"))))),
                (fun _ loc -> 
# 1031 "vernac/g_vernac.mlg"
            user_err Pp.(str "Print Modules is obsolete; use Print Libraries instead") 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("LoadPath"))))),
                             (Extend.Aopt (Extend.Aentry dirpath))),
                (fun dir _ loc -> 
# 1029 "vernac/g_vernac.mlg"
                                                 PrintLoadPath dir 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Custom"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Grammar"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun ent _ _ loc -> 
# 1028 "vernac/g_vernac.mlg"
            PrintCustomGrammar ent 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Grammar"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun ent _ loc -> 
# 1025 "vernac/g_vernac.mlg"
            PrintGrammar ent 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Section"))))),
                             (Extend.Aentry global)),
                (fun s _ loc -> 
# 1022 "vernac/g_vernac.mlg"
                                         PrintSectionContext s 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("All"))))),
                (fun _ loc -> 
# 1021 "vernac/g_vernac.mlg"
                         PrintFullContext 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Term"))))),
                                          (Extend.Aentry smart_global)),
                             (Extend.Aopt (Extend.Aentry univ_name_list))),
                (fun l qid _ loc -> 
# 1020 "vernac/g_vernac.mlg"
                                                                      PrintName (qid,l) 
                                    ))])])
        in let () =
        Pcoq.grammar_extend printunivs_subgraph None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Subgraph"))))),
                                                        (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                           (Extend.Alist0 (Extend.Aentry reference))),
                              (Extend.Atoken (Tok.PKEYWORD (")")))),
                 (fun _ l _ _ loc -> 
# 1069 "vernac/g_vernac.mlg"
                                                             l 
                                     ))])])
        in let () =
        Pcoq.grammar_extend class_rawexpr None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry smart_global)),
                 (fun qid loc -> 
# 1074 "vernac/g_vernac.mlg"
                                RefClass qid 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Sortclass"))))),
                (fun _ loc -> 
# 1073 "vernac/g_vernac.mlg"
                               SortClass 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Funclass"))))),
                (fun _ loc -> 
# 1072 "vernac/g_vernac.mlg"
                              FunClass 
                              ))])])
        in let () =
        Pcoq.grammar_extend locatable None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("Module"))))),
                              (Extend.Aentry global)),
                 (fun qid _ loc -> 
# 1081 "vernac/g_vernac.mlg"
                                          LocateModule qid 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Library"))))),
                             (Extend.Aentry global)),
                (fun qid _ loc -> 
# 1080 "vernac/g_vernac.mlg"
                                           LocateLibrary qid 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("File"))))),
                             (Extend.Aentry ne_string)),
                (fun f _ loc -> 
# 1079 "vernac/g_vernac.mlg"
                                         LocateFile f 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Term"))))),
                             (Extend.Aentry smart_global)),
                (fun qid _ loc -> 
# 1078 "vernac/g_vernac.mlg"
                                              LocateTerm qid 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry smart_global)),
                (fun qid loc -> 
# 1077 "vernac/g_vernac.mlg"
                                LocateAny qid 
                                ))])])
        in let () =
        Pcoq.grammar_extend option_setting None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PSTRING (None)))),
                 (fun s loc -> 
# 1086 "vernac/g_vernac.mlg"
                            OptionSetString s 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry integer)),
                (fun n loc -> 
# 1085 "vernac/g_vernac.mlg"
                            OptionSetInt n 
                              ));
                Extend.Rule (Extend.Stop, (fun loc -> 
# 1084 "vernac/g_vernac.mlg"
             OptionSetTrue 
                                                      ))])])
        in let () =
        Pcoq.grammar_extend option_ref_value None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Atoken (Tok.PSTRING (None)))),
                 (fun s loc -> 
# 1090 "vernac/g_vernac.mlg"
                           StringRefValue s 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry global)),
                (fun id loc -> 
# 1089 "vernac/g_vernac.mlg"
                           QualidRefValue id 
                               ))])])
        in let () =
        Pcoq.grammar_extend option_table None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Alist1 (Extend.Arules [Extend.Rules 
                                                            (Extend.NextNoRec 
                                                            (Extend.Stop,
                                                            (Extend.Atoken (Tok.PIDENT (None)))),
                                                            (fun x loc -> 
                                                            
# 1093 "vernac/g_vernac.mlg"
                                    x 
                                                            ))]))),
                 (fun fl loc -> 
# 1093 "vernac/g_vernac.mlg"
                                               fl 
                                ))])])
        in let () =
        Pcoq.grammar_extend as_dirpath None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop,
                              (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                          (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("as")))),
                                                          (Extend.Aentry dirpath)),
                                                          (fun d _ loc -> 
                                                          
# 1096 "vernac/g_vernac.mlg"
                                         d 
                                                          ))]))),
                 (fun d loc -> 
# 1096 "vernac/g_vernac.mlg"
                                                    d 
                               ))])])
        in let () =
        Pcoq.grammar_extend ne_in_or_out_modules None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("outside"))))),
                              (Extend.Alist1 (Extend.Aentry global))),
                 (fun l _ loc -> 
# 1100 "vernac/g_vernac.mlg"
                                               SearchOutside l 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("inside"))))),
                             (Extend.Alist1 (Extend.Aentry global))),
                (fun l _ loc -> 
# 1099 "vernac/g_vernac.mlg"
                                              SearchInside l 
                                ))])])
        in let () =
        Pcoq.grammar_extend in_or_out_modules None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 1104 "vernac/g_vernac.mlg"
             SearchOutside [] 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Aentry ne_in_or_out_modules)),
                (fun m loc -> 
# 1103 "vernac/g_vernac.mlg"
                                      m 
                              ))])])
        in let () =
        Pcoq.grammar_extend comment None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Stop, (Extend.Aentry natural)),
                 (fun n loc -> 
# 1109 "vernac/g_vernac.mlg"
                         CommentInt n 
                               ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PSTRING (None)))),
                (fun s loc -> 
# 1108 "vernac/g_vernac.mlg"
                        CommentString s 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Aentry constr)),
                (fun c loc -> 
# 1107 "vernac/g_vernac.mlg"
                        CommentConstr c 
                              ))])])
        in let () =
        Pcoq.grammar_extend positive_search_mark None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 1112 "vernac/g_vernac.mlg"
                                true 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Atoken (Tok.PKEYWORD ("-")))),
                (fun _ loc -> 
# 1112 "vernac/g_vernac.mlg"
                 false 
                              ))])])
        in let () =
        Pcoq.grammar_extend scope None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("%")))),
                              (Extend.Atoken (Tok.PIDENT (None)))),
                 (fun key _ loc -> 
# 1115 "vernac/g_vernac.mlg"
                              key 
                                   ))])])
        in let () =
        Pcoq.grammar_extend searchabout_query None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Aentry positive_search_mark)),
                              (Extend.Aentry constr_pattern)),
                 (fun p b loc -> 
# 1121 "vernac/g_vernac.mlg"
              (b, SearchSubPattern p) 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Aentry positive_search_mark)),
                                          (Extend.Aentry ne_string)),
                             (Extend.Aopt (Extend.Aentry scope))),
                (fun sc s b loc -> 
# 1119 "vernac/g_vernac.mlg"
              (b, SearchString (s,sc)) 
                                   ))])])
        in let () =
        Pcoq.grammar_extend searchabout_queries None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 1128 "vernac/g_vernac.mlg"
             ([],SearchOutside []) 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Aentry searchabout_query)),
                             (Extend.Aentry searchabout_queries)),
                (fun l s loc -> 
# 1127 "vernac/g_vernac.mlg"
          let (sl,m) = l in (s::sl,m) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop,
                             (Extend.Aentry ne_in_or_out_modules)),
                (fun m loc -> 
# 1125 "vernac/g_vernac.mlg"
                                      ([],m) 
                              ))])])
        in let () =
        Pcoq.grammar_extend univ_name_list None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("@{")))),
                                           (Extend.Alist0 (Extend.Aentry name))),
                              (Extend.Atoken (Tok.PKEYWORD ("}")))),
                 (fun _ l _ loc -> 
# 1132 "vernac/g_vernac.mlg"
                                         l 
                                   ))])])
        in ()

let _ = let () =
        Pcoq.grammar_extend command None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Declare"))))),
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Custom"))))),
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("Entry"))))),
                              (Extend.Atoken (Tok.PIDENT (None)))),
                 (fun s _ _ _ loc -> 
# 1169 "vernac/g_vernac.mlg"
             VernacDeclareCustomEntry s 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Declare"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Reduction"))))),
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                          (Extend.Atoken (Tok.PKEYWORD (":=")))),
                             (Extend.Aentry red_expr)),
                (fun r _ s _ _ loc -> 
# 1164 "vernac/g_vernac.mlg"
             VernacDeclareReduction (s,r) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Debug"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("Off"))))),
                (fun _ _ loc -> 
# 1158 "vernac/g_vernac.mlg"
            VernacSetOption (false, ["Ltac";"Debug"], OptionUnset) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Debug"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("On"))))),
                (fun _ _ loc -> 
# 1155 "vernac/g_vernac.mlg"
            VernacSetOption (false, ["Ltac";"Debug"], OptionSetTrue) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Back"))))),
                             (Extend.Aentry natural)),
                (fun n _ loc -> 
# 1151 "vernac/g_vernac.mlg"
                                       VernacBack n 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("Back"))))),
                (fun _ loc -> 
# 1150 "vernac/g_vernac.mlg"
                          VernacBack 1 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Reset"))))),
                             (Extend.Aentry identref)),
                (fun id _ loc -> 
# 1149 "vernac/g_vernac.mlg"
                                          VernacResetName id 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Reset"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("Initial"))))),
                (fun _ _ loc -> 
# 1148 "vernac/g_vernac.mlg"
                                            VernacResetInitial 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Restore"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("State"))))),
                             (Extend.Aentry ne_string)),
                (fun s _ _ loc -> 
# 1145 "vernac/g_vernac.mlg"
                                                           VernacRestoreState s 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Restore"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("State"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun s _ _ loc -> 
# 1144 "vernac/g_vernac.mlg"
                                                       VernacRestoreState s 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Write"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("State"))))),
                             (Extend.Aentry ne_string)),
                (fun s _ _ loc -> 
# 1143 "vernac/g_vernac.mlg"
                                                         VernacWriteState s 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Write"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("State"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun s _ _ loc -> 
# 1142 "vernac/g_vernac.mlg"
                                                     VernacWriteState s 
                                  ))])])
        in ()

let _ = let only_parsing = Pcoq.Entry.create "only_parsing"
        and level = Pcoq.Entry.create "level"
        and syntax_modifier = Pcoq.Entry.create "syntax_modifier"
        and syntax_extension_type = Pcoq.Entry.create "syntax_extension_type"
        and at_level = Pcoq.Entry.create "at_level"
        and constr_as_binder_kind = Pcoq.Entry.create "constr_as_binder_kind"
        in
        let () =
        Pcoq.grammar_extend syntax None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Reserved"))))),
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("Notation"))))),
                                           (Extend.Aentry ne_lstring)),
                              (Extend.Arules [Extend.Rules (Extend.Stop, (fun
                                                           loc -> 
# 1217 "vernac/g_vernac.mlg"
                                                                           [] 
                                                                  ));
                                             Extend.Rules (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.NextNoRec 
                                                          (Extend.Stop,
                                                          (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                          (Extend.Alist1sep ((Extend.Aentry syntax_modifier), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                          (Extend.Atoken (Tok.PKEYWORD (")")))),
                                                          (fun _ l _ loc ->
                                                          
# 1217 "vernac/g_vernac.mlg"
                                                                l 
                                                          ))])),
                 (fun l s _ _ loc -> 
# 1218 "vernac/g_vernac.mlg"
              VernacSyntaxExtension (false, (s,l)) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Reserved"))))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Infix"))))),
                                          (Extend.Aentry ne_lstring)),
                             (Extend.Arules [Extend.Rules (Extend.Stop, (fun
                                                          loc -> 
# 1211 "vernac/g_vernac.mlg"
                                                                           [] 
                                                                 ));
                                            Extend.Rules (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                         (Extend.Alist1sep ((Extend.Aentry syntax_modifier), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                         (Extend.Atoken (Tok.PKEYWORD (")")))),
                                                         (fun _ l _ loc -> 
                                                         
# 1211 "vernac/g_vernac.mlg"
                                                                l 
                                                         ))])),
                (fun l s _ _ loc -> 
# 1212 "vernac/g_vernac.mlg"
             let s = CAst.map (fun s -> "x '"^s^"' y") s in
           VernacSyntaxExtension (true,(s,l)) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Format"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Notation"))))),
                                                       (Extend.Atoken (Tok.PSTRING (None)))),
                                          (Extend.Atoken (Tok.PSTRING (None)))),
                             (Extend.Atoken (Tok.PSTRING (None)))),
                (fun fmt s n _ _ loc -> 
# 1208 "vernac/g_vernac.mlg"
             VernacNotationAddFormat (n,s,fmt) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Notation"))))),
                                                                    (Extend.Aentry lstring)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                                       (Extend.Aentry constr)),
                                          (Extend.Arules [Extend.Rules 
                                                         (Extend.Stop, (fun
                                                         loc -> 
# 1204 "vernac/g_vernac.mlg"
                                                                              [] 
                                                                ));
                                                         Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                         (Extend.Alist1sep ((Extend.Aentry syntax_modifier), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                         (Extend.Atoken (Tok.PKEYWORD (")")))),
                                                         (fun _ l _ loc -> 
                                                         
# 1204 "vernac/g_vernac.mlg"
                                                                   l 
                                                         ))])),
                             (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                         (Extend.Atoken (Tok.PIDENT (None)))),
                                                         (fun sc _ loc -> 
                                                         
# 1205 "vernac/g_vernac.mlg"
                                         sc 
                                                         ))]))),
                (fun sc modl c _ s _ loc -> 
# 1206 "vernac/g_vernac.mlg"
             VernacNotation (c,(s,modl),sc) 
                                            ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Notation"))))),
                                                                    (Extend.Aentry identref)),
                                                                    (Extend.Alist0 (Extend.Aentry ident))),
                                                       (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                          (Extend.Aentry constr)),
                             (Extend.Aentry only_parsing)),
                (fun b c _ idl id _ loc -> 
# 1200 "vernac/g_vernac.mlg"
             VernacSyntacticDefinition
             (id,(idl,c),b) 
                                           ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Infix"))))),
                                                                    (Extend.Aentry ne_lstring)),
                                                                    (Extend.Atoken (Tok.PKEYWORD (":=")))),
                                                       (Extend.Aentry constr)),
                                          (Extend.Arules [Extend.Rules 
                                                         (Extend.Stop, (fun
                                                         loc -> 
# 1195 "vernac/g_vernac.mlg"
                                                                              [] 
                                                                ));
                                                         Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                         (Extend.Alist1sep ((Extend.Aentry syntax_modifier), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                                         (Extend.Atoken (Tok.PKEYWORD (")")))),
                                                         (fun _ l _ loc -> 
                                                         
# 1195 "vernac/g_vernac.mlg"
                                                                   l 
                                                         ))])),
                             (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PKEYWORD (":")))),
                                                         (Extend.Atoken (Tok.PIDENT (None)))),
                                                         (fun sc _ loc -> 
                                                         
# 1196 "vernac/g_vernac.mlg"
                                         sc 
                                                         ))]))),
                (fun sc modl p _ op _ loc -> 
# 1197 "vernac/g_vernac.mlg"
           VernacInfix ((op,modl),p,sc) 
                                             ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Bind"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Scope"))))),
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                          (Extend.Atoken (Tok.PKEYWORD ("with")))),
                             (Extend.Alist1 (Extend.Aentry class_rawexpr))),
                (fun refl _ sc _ _ loc -> 
# 1192 "vernac/g_vernac.mlg"
                                       VernacBindScope (sc,refl) 
                                          ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Undelimit"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Scope"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun sc _ _ loc -> 
# 1189 "vernac/g_vernac.mlg"
           VernacDelimiters (sc, None) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Delimit"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("Scope"))))),
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                          (Extend.Atoken (Tok.PKEYWORD ("with")))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun key _ sc _ _ loc -> 
# 1187 "vernac/g_vernac.mlg"
           VernacDelimiters (sc, Some key) 
                                         ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Close"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Scope"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun sc _ _ loc -> 
# 1184 "vernac/g_vernac.mlg"
           VernacOpenCloseScope (false,sc) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("Open"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("Scope"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun sc _ _ loc -> 
# 1181 "vernac/g_vernac.mlg"
           VernacOpenCloseScope (true,sc) 
                                   ))])])
        in let () =
        Pcoq.grammar_extend only_parsing None
        (None, [(None, None,
                [Extend.Rule (Extend.Stop, (fun loc -> 
# 1229 "vernac/g_vernac.mlg"
             None 
                                                       ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("compat"))))),
                                          (Extend.Atoken (Tok.PSTRING (None)))),
                             (Extend.Atoken (Tok.PKEYWORD (")")))),
                (fun _ s _ _ loc -> 
# 1228 "vernac/g_vernac.mlg"
           Some (parse_compat_version s) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("(")))),
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("only"))))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("parsing"))))),
                             (Extend.Atoken (Tok.PKEYWORD (")")))),
                (fun _ _ _ _ loc -> 
# 1226 "vernac/g_vernac.mlg"
           Some Flags.Current 
                                    ))])])
        in let () =
        Pcoq.grammar_extend level None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("next"))))),
                              (Extend.Atoken (Tok.PIDENT (Some ("level"))))),
                 (fun _ _ loc -> 
# 1233 "vernac/g_vernac.mlg"
                                         NextLevel 
                                 ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("level"))))),
                             (Extend.Aentry natural)),
                (fun n _ loc -> 
# 1232 "vernac/g_vernac.mlg"
                                        NumLevel n 
                                ))])])
        in let () =
        Pcoq.grammar_extend syntax_modifier None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PIDENT (None)))),
                              (Extend.Aentry syntax_extension_type)),
                 (fun typ x loc -> 
# 1258 "vernac/g_vernac.mlg"
                                                    SetEntryType (x,typ) 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (None)))),
                             (Extend.Aentry constr_as_binder_kind)),
                (fun b x loc -> 
# 1257 "vernac/g_vernac.mlg"
                                                  SetItemLevel ([x],Some b,None) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (None)))),
                                                       (Extend.Atoken (Tok.PKEYWORD ("at")))),
                                          (Extend.Aentry level)),
                             (Extend.Aentry constr_as_binder_kind)),
                (fun b lev _ x loc -> 
# 1256 "vernac/g_vernac.mlg"
          SetItemLevel ([x],Some b,Some lev) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                          (Extend.Atoken (Tok.PKEYWORD ("at")))),
                             (Extend.Aentry level)),
                (fun lev _ x loc -> 
# 1254 "vernac/g_vernac.mlg"
                                          SetItemLevel ([x],None,Some lev) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (None)))),
                                                                    (Extend.Atoken (Tok.PKEYWORD (",")))),
                                                       (Extend.Alist1sep ((Extend.Arules 
                                                       [Extend.Rules 
                                                       (Extend.NextNoRec 
                                                       (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (None)))),
                                                       (fun id loc -> 
                                                       
# 1252 "vernac/g_vernac.mlg"
                                                   id 
                                                       ))]), (Extend.Atoken (Tok.PKEYWORD (",")))))),
                                          (Extend.Atoken (Tok.PKEYWORD ("at")))),
                             (Extend.Aentry level)),
                (fun lev _ l _ x loc -> 
# 1253 "vernac/g_vernac.mlg"
                         SetItemLevel (x::l,None,Some lev) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("format"))))),
                                          (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PSTRING (None)))),
                                                         (fun s loc -> 
                                                         
# 1247 "vernac/g_vernac.mlg"
                                              CAst.make ~loc s 
                                                         ))])),
                             (Extend.Aopt (Extend.Arules [Extend.Rules 
                                                         (Extend.NextNoRec 
                                                         (Extend.Stop,
                                                         (Extend.Atoken (Tok.PSTRING (None)))),
                                                         (fun s loc -> 
                                                         
# 1248 "vernac/g_vernac.mlg"
                                                  CAst.make ~loc s 
                                                         ))]))),
                (fun s2 s1 _ loc -> 
# 1249 "vernac/g_vernac.mlg"
            begin match s1, s2 with
          | { CAst.v = k }, Some s -> SetFormat(k,s)
          | s, None -> SetFormat ("text",s) end 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("compat"))))),
                             (Extend.Atoken (Tok.PSTRING (None)))),
                (fun s _ loc -> 
# 1246 "vernac/g_vernac.mlg"
          SetCompatVersion (parse_compat_version s) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("only"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("parsing"))))),
                (fun _ _ loc -> 
# 1244 "vernac/g_vernac.mlg"
                                           SetOnlyParsing 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("only"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("printing"))))),
                (fun _ _ loc -> 
# 1243 "vernac/g_vernac.mlg"
                                            SetOnlyPrinting 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("no"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("associativity"))))),
                (fun _ _ loc -> 
# 1242 "vernac/g_vernac.mlg"
                                               SetAssoc Gramlib.Gramext.NonA 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("right"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("associativity"))))),
                (fun _ _ loc -> 
# 1241 "vernac/g_vernac.mlg"
                                                  SetAssoc Gramlib.Gramext.RightA 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("left"))))),
                             (Extend.Atoken (Tok.PIDENT (Some
                             ("associativity"))))),
                (fun _ _ loc -> 
# 1240 "vernac/g_vernac.mlg"
                                                 SetAssoc Gramlib.Gramext.LeftA 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PKEYWORD ("in")))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("custom"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (None)))),
                                                       (Extend.Atoken (Tok.PKEYWORD ("at")))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("level"))))),
                             (Extend.Aentry natural)),
                (fun n _ _ x _ _ loc -> 
# 1239 "vernac/g_vernac.mlg"
           SetCustomEntry (x,Some n) 
                                        ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("in")))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("custom"))))),
                             (Extend.Atoken (Tok.PIDENT (None)))),
                (fun x _ _ loc -> 
# 1237 "vernac/g_vernac.mlg"
                                             SetCustomEntry (x,None) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PKEYWORD ("at")))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("level"))))),
                             (Extend.Aentry natural)),
                (fun n _ _ loc -> 
# 1236 "vernac/g_vernac.mlg"
                                              SetLevel n 
                                  ))])])
        in let () =
        Pcoq.grammar_extend syntax_extension_type None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                        (Extend.Stop,
                                                        (Extend.Atoken (Tok.PIDENT (Some
                                                        ("custom"))))),
                                                        (Extend.Atoken (Tok.PIDENT (None)))),
                                           (Extend.Aopt (Extend.Aentry at_level))),
                              (Extend.Aopt (Extend.Aentry constr_as_binder_kind))),
                 (fun b n x _ loc -> 
# 1273 "vernac/g_vernac.mlg"
             ETConstr (InCustomEntry x,b,n) 
                                     ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("closed"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("binder"))))),
                (fun _ _ loc -> 
# 1271 "vernac/g_vernac.mlg"
                                            ETBinder false 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Next 
                                                                    (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("strict"))))),
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("pattern"))))),
                                                       (Extend.Atoken (Tok.PKEYWORD ("at")))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("level"))))),
                             (Extend.Aentry natural)),
                (fun n _ _ _ _ loc -> 
# 1270 "vernac/g_vernac.mlg"
                                                                               ETPattern (true,Some n) 
                                      ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("strict"))))),
                             (Extend.Atoken (Tok.PIDENT (Some ("pattern"))))),
                (fun _ _ loc -> 
# 1269 "vernac/g_vernac.mlg"
                                             ETPattern (true,None) 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                                    (Extend.Atoken (Tok.PIDENT (Some
                                                                    ("pattern"))))),
                                                       (Extend.Atoken (Tok.PKEYWORD ("at")))),
                                          (Extend.Atoken (Tok.PIDENT (Some
                                          ("level"))))),
                             (Extend.Aentry natural)),
                (fun n _ _ _ loc -> 
# 1268 "vernac/g_vernac.mlg"
                                                               ETPattern (false,Some n) 
                                    ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("pattern"))))),
                (fun _ loc -> 
# 1267 "vernac/g_vernac.mlg"
                             ETPattern (false,None) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                       (Extend.Atoken (Tok.PIDENT (Some
                                                       ("constr"))))),
                                          (Extend.Aopt (Extend.Aentry at_level))),
                             (Extend.Aopt (Extend.Aentry constr_as_binder_kind))),
                (fun b n _ loc -> 
# 1266 "vernac/g_vernac.mlg"
                                                                             ETConstr (InConstrEntry,b,n) 
                                  ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("constr"))))),
                (fun _ loc -> 
# 1265 "vernac/g_vernac.mlg"
                            ETConstr (InConstrEntry,None,None) 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("binder"))))),
                (fun _ loc -> 
# 1264 "vernac/g_vernac.mlg"
                            ETBinder true 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("bigint"))))),
                (fun _ loc -> 
# 1263 "vernac/g_vernac.mlg"
                            ETBigint 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("global"))))),
                (fun _ loc -> 
# 1262 "vernac/g_vernac.mlg"
                                                           ETGlobal 
                              ));
                Extend.Rule
                (Extend.Next (Extend.Stop, (Extend.Atoken (Tok.PIDENT (Some
                             ("ident"))))),
                (fun _ loc -> 
# 1262 "vernac/g_vernac.mlg"
                           ETIdent 
                              ))])])
        in let () =
        Pcoq.grammar_extend at_level None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Stop,
                                           (Extend.Atoken (Tok.PKEYWORD ("at")))),
                              (Extend.Aentry level)),
                 (fun n _ loc -> 
# 1277 "vernac/g_vernac.mlg"
                             n 
                                 ))])])
        in let () =
        Pcoq.grammar_extend constr_as_binder_kind None
        (None, [(None, None,
                [Extend.Rule
                 (Extend.Next (Extend.Next (Extend.Next (Extend.Stop,
                                                        (Extend.Atoken (Tok.PKEYWORD ("as")))),
                                           (Extend.Atoken (Tok.PIDENT (Some
                                           ("strict"))))),
                              (Extend.Atoken (Tok.PIDENT (Some
                              ("pattern"))))),
                 (fun _ _ _ loc -> 
# 1282 "vernac/g_vernac.mlg"
                                                   Notation_term.AsStrictPattern 
                                   ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("as")))),
                             (Extend.Atoken (Tok.PIDENT (Some ("pattern"))))),
                (fun _ _ loc -> 
# 1281 "vernac/g_vernac.mlg"
                                   Notation_term.AsIdentOrPattern 
                                ));
                Extend.Rule
                (Extend.Next (Extend.Next (Extend.Stop,
                                          (Extend.Atoken (Tok.PKEYWORD ("as")))),
                             (Extend.Atoken (Tok.PIDENT (Some ("ident"))))),
                (fun _ _ loc -> 
# 1280 "vernac/g_vernac.mlg"
                                 Notation_term.AsIdent 
                                ))])])
        in ()

